package com.prolog.cs.biz.inbound.dto.failure;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * @Description 收货异常日志 数据传输对象
 * @Author Xiong
 * @Date 2021-04-02
 */

@ApiModel
public class ExceptionLogDTO implements Serializable {
	private static final long serialVersionUID = 4568865395595120996L;
	@ApiModelProperty(value = "错误行日志ID(主键雪花id)")
	// @NotBlank(message = "错误行日志ID(主键雪花id) can not blank!", groups = validEdit.class)
	private String exceptionLogId;

	@ApiModelProperty(value = "错误行ID", required = true, position = -1)
	// @NotBlank(message = "错误行ID can not blank!", groups = validAdd.class)
	private String exceptionLineId;

	@ApiModelProperty(value = "错误单头ID", required = true, position = -1)
	// @NotBlank(message = "错误单头ID can not blank!", groups = validAdd.class)
	private String exceptionHeaderId;

	@ApiModelProperty(value = "收货行ID", required = true, position = -1)
	// @NotBlank(message = "行ID can not blank!", groups = validAdd.class)
	private String rcvLineId;

	@ApiModelProperty(value = "收货头ID", required = true, position = -1)
	// @NotBlank(message = "头ID can not blank!", groups = validAdd.class)
	private String rcvHeaderId;

	@ApiModelProperty(value = "行号", required = true, position = -1)
	// @NotNull(message = "行号 can not blank!", groups = validAdd.class)
	private BigDecimal lineNum;

	@ApiModelProperty(value = "货主ID", required = true, position = -1)
	// @NotBlank(message = "货主ID can not blank!", groups = validAdd.class)
	private String mtlOwnerId;

	@ApiModelProperty(value = "仓库ID", required = true, position = -1)
	// @NotBlank(message = "仓库ID can not blank!", groups = validAdd.class)
	private String warehouseId;

	@ApiModelProperty(value = "商品ID", required = true, position = -1)
	// @NotBlank(message = "商品ID can not blank!", groups = validAdd.class)
	private String itemId;

	@ApiModelProperty(value = "旧批号ID", required = true, position = -1)
	// @NotBlank(message = "旧批号ID can not blank!", groups = validAdd.class)
	private String batchIdOld;

	@ApiModelProperty(value = "旧批号编码", required = true, position = -1)
	// @NotBlank(message = "旧批号编码 can not blank!", groups = validAdd.class)
	private String batchNumOld;

	@ApiModelProperty(value = "旧生产日期", required = true, position = -1)
	// @NotNull(message = "旧生产日期 can not blank!", groups = validAdd.class)
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date productionDateOld;

	@ApiModelProperty(value = "旧有效期至", required = true, position = -1)
	// @NotNull(message = "旧有效期至 can not blank!", groups = validAdd.class)
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date expiredDateOld;

	@ApiModelProperty(value = "旧数量", required = true, position = -1)
	// @NotNull(message = "旧数量 can not blank!", groups = validAdd.class)
	private BigDecimal qtyOld;

	@ApiModelProperty(value = "旧数量(最小单位)", required = true, position = -1)
	// @NotNull(message = "旧数量(最小单位) can not blank!", groups = validAdd.class)
	private BigDecimal pieceQtyOld;

	@ApiModelProperty(value = "包装单位", required = true, position = -1)
	// @NotBlank(message = "包装单位 can not blank!", groups = validAdd.class)
	private String uom;

	@ApiModelProperty(value = "包装含量", required = true, position = -1)
	// @NotNull(message = "包装含量 can not blank!", groups = validAdd.class)
	private BigDecimal conversionRate;

	@ApiModelProperty(value = "错误发现节点（10收货错误/20质检错误/30复检错误/40上架错误）", required = true, position = -1)
	// @NotBlank(message = "错误发现节点不可为空!", groups = validAdd.class)
	private String nodeType;

	@ApiModelProperty(value = "错误节点的任务id", required = true, position = -1)
	// @NotBlank(message = "错误节点的任务id不可为空!", groups = validAdd.class)
	private String nodeTaskId;

	@ApiModelProperty(value = "旧供应商批次号")
	private String vendorBatchNumOld;

	@ApiModelProperty(value = "旧生产商批号")
	private String manufacturerLotNumOld;

	@ApiModelProperty(value = "容器编码")
	private String containerCode;

	@ApiModelProperty(value = "货格编码")
	private String goodsCaseCode;

	@ApiModelProperty(value = "新批号")
	private String batchIdNew;

	@ApiModelProperty(value = "新批号编码")
	private String batchNumNew;

	@ApiModelProperty(value = "新生产日期")
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date productionDateNew;

	@ApiModelProperty(value = "新有效期至")
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date expiredDateNew;

	@ApiModelProperty(value = "新供应商批次号")
	private String vendorBatchNumNew;

	@ApiModelProperty(value = "新生产商批号")
	private String manufacturerLotNumNew;

	@ApiModelProperty(value = "新数量")
	private BigDecimal qtyNew;

	@ApiModelProperty(value = "包装单位类型(2019/2020)新老包装")
	private String uomType;

	@ApiModelProperty(value = "新数量（最小单位）")
	private BigDecimal pieceQtyNew;

	@ApiModelProperty(value = "执行者（异常处理人）")
	private String executorUser;

	@ApiModelProperty(value = "操作时间")
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date executorDate;

	@ApiModelProperty(value = "来源单类型")
	private String srcBillType;

	@ApiModelProperty(value = "来源单头ID(PO/XT/YC)")
	private String srcHeaderId;

	@ApiModelProperty(value = "来源单ID")
	private String srcLineId;

	@ApiModelProperty(value = "ASN行ID")
	private String asnHeaderId;

	@ApiModelProperty(value = "ASN行ID")
	private String asnLineId;

	@ApiModelProperty(value = "新的显示生产日期", position = -4)
	private String showProductionDateNew;

	@ApiModelProperty(value = "新的显示有效期至", position = -4)
	private String showExpiredDateNew;

	@ApiModelProperty(value = "旧的显示生产日期", position = -4)
	private String showProductionDateOld;

	@ApiModelProperty(value = "旧的显示有效期至", position = -4)
	private String showExpiredDateOld;

	public String getShowProductionDateNew() {
		return showProductionDateNew;
	}

	public void setShowProductionDateNew(String showProductionDateNew) {
		this.showProductionDateNew = showProductionDateNew;
	}

	public String getShowExpiredDateNew() {
		return showExpiredDateNew;
	}

	public void setShowExpiredDateNew(String showExpiredDateNew) {
		this.showExpiredDateNew = showExpiredDateNew;
	}

	public String getShowProductionDateOld() {
		return showProductionDateOld;
	}

	public void setShowProductionDateOld(String showProductionDateOld) {
		this.showProductionDateOld = showProductionDateOld;
	}

	public String getShowExpiredDateOld() {
		return showExpiredDateOld;
	}

	public void setShowExpiredDateOld(String showExpiredDateOld) {
		this.showExpiredDateOld = showExpiredDateOld;
	}

	public String getExceptionLogId() {
		return exceptionLogId;
	}

	public void setExceptionLogId(String exceptionLogId) {
		this.exceptionLogId = exceptionLogId;
	}

	public String getExceptionLineId() {
		return exceptionLineId;
	}

	public void setExceptionLineId(String exceptionLineId) {
		this.exceptionLineId = exceptionLineId;
	}

	public String getExceptionHeaderId() {
		return exceptionHeaderId;
	}

	public void setExceptionHeaderId(String exceptionHeaderId) {
		this.exceptionHeaderId = exceptionHeaderId;
	}

	public String getRcvLineId() {
		return rcvLineId;
	}

	public void setRcvLineId(String rcvLineId) {
		this.rcvLineId = rcvLineId;
	}

	public String getRcvHeaderId() {
		return rcvHeaderId;
	}

	public void setRcvHeaderId(String rcvHeaderId) {
		this.rcvHeaderId = rcvHeaderId;
	}

	public BigDecimal getLineNum() {
		return lineNum;
	}

	public void setLineNum(BigDecimal lineNum) {
		this.lineNum = lineNum;
	}

	public String getMtlOwnerId() {
		return mtlOwnerId;
	}

	public void setMtlOwnerId(String mtlOwnerId) {
		this.mtlOwnerId = mtlOwnerId;
	}

	public String getWarehouseId() {
		return warehouseId;
	}

	public void setWarehouseId(String warehouseId) {
		this.warehouseId = warehouseId;
	}

	public String getItemId() {
		return itemId;
	}

	public void setItemId(String itemId) {
		this.itemId = itemId;
	}

	public String getBatchIdOld() {
		return batchIdOld;
	}

	public void setBatchIdOld(String batchIdOld) {
		this.batchIdOld = batchIdOld;
	}

	public String getBatchNumOld() {
		return batchNumOld;
	}

	public void setBatchNumOld(String batchNumOld) {
		this.batchNumOld = batchNumOld;
	}

	public Date getProductionDateOld() {
		return productionDateOld;
	}

	public void setProductionDateOld(Date productionDateOld) {
		this.productionDateOld = productionDateOld;
	}

	public Date getExpiredDateOld() {
		return expiredDateOld;
	}

	public void setExpiredDateOld(Date expiredDateOld) {
		this.expiredDateOld = expiredDateOld;
	}

	public BigDecimal getQtyOld() {
		return qtyOld;
	}

	public void setQtyOld(BigDecimal qtyOld) {
		this.qtyOld = qtyOld;
	}

	public BigDecimal getPieceQtyOld() {
		return pieceQtyOld;
	}

	public void setPieceQtyOld(BigDecimal pieceQtyOld) {
		this.pieceQtyOld = pieceQtyOld;
	}

	public String getUom() {
		return uom;
	}

	public void setUom(String uom) {
		this.uom = uom;
	}

	public BigDecimal getConversionRate() {
		return conversionRate;
	}

	public void setConversionRate(BigDecimal conversionRate) {
		this.conversionRate = conversionRate;
	}

	public String getNodeType() {
		return nodeType;
	}

	public void setNodeType(String nodeType) {
		this.nodeType = nodeType;
	}

	public String getNodeTaskId() {
		return nodeTaskId;
	}

	public void setNodeTaskId(String nodeTaskId) {
		this.nodeTaskId = nodeTaskId;
	}

	public String getVendorBatchNumOld() {
		return vendorBatchNumOld;
	}

	public void setVendorBatchNumOld(String vendorBatchNumOld) {
		this.vendorBatchNumOld = vendorBatchNumOld;
	}

	public String getManufacturerLotNumOld() {
		return manufacturerLotNumOld;
	}

	public void setManufacturerLotNumOld(String manufacturerLotNumOld) {
		this.manufacturerLotNumOld = manufacturerLotNumOld;
	}

	public String getContainerCode() {
		return containerCode;
	}

	public void setContainerCode(String containerCode) {
		this.containerCode = containerCode;
	}

	public String getGoodsCaseCode() {
		return goodsCaseCode;
	}

	public void setGoodsCaseCode(String goodsCaseCode) {
		this.goodsCaseCode = goodsCaseCode;
	}

	public String getBatchIdNew() {
		return batchIdNew;
	}

	public void setBatchIdNew(String batchIdNew) {
		this.batchIdNew = batchIdNew;
	}

	public String getBatchNumNew() {
		return batchNumNew;
	}

	public void setBatchNumNew(String batchNumNew) {
		this.batchNumNew = batchNumNew;
	}

	public Date getProductionDateNew() {
		return productionDateNew;
	}

	public void setProductionDateNew(Date productionDateNew) {
		this.productionDateNew = productionDateNew;
	}

	public Date getExpiredDateNew() {
		return expiredDateNew;
	}

	public void setExpiredDateNew(Date expiredDateNew) {
		this.expiredDateNew = expiredDateNew;
	}

	public String getVendorBatchNumNew() {
		return vendorBatchNumNew;
	}

	public void setVendorBatchNumNew(String vendorBatchNumNew) {
		this.vendorBatchNumNew = vendorBatchNumNew;
	}

	public String getManufacturerLotNumNew() {
		return manufacturerLotNumNew;
	}

	public void setManufacturerLotNumNew(String manufacturerLotNumNew) {
		this.manufacturerLotNumNew = manufacturerLotNumNew;
	}

	public BigDecimal getQtyNew() {
		return qtyNew;
	}

	public void setQtyNew(BigDecimal qtyNew) {
		this.qtyNew = qtyNew;
	}

	public String getUomType() {
		return uomType;
	}

	public void setUomType(String uomType) {
		this.uomType = uomType;
	}

	public BigDecimal getPieceQtyNew() {
		return pieceQtyNew;
	}

	public void setPieceQtyNew(BigDecimal pieceQtyNew) {
		this.pieceQtyNew = pieceQtyNew;
	}

	public String getExecutorUser() {
		return executorUser;
	}

	public void setExecutorUser(String executorUser) {
		this.executorUser = executorUser;
	}

	public Date getExecutorDate() {
		return executorDate;
	}

	public void setExecutorDate(Date executorDate) {
		this.executorDate = executorDate;
	}

	public String getSrcBillType() {
		return srcBillType;
	}

	public void setSrcBillType(String srcBillType) {
		this.srcBillType = srcBillType;
	}

	public String getSrcHeaderId() {
		return srcHeaderId;
	}

	public void setSrcHeaderId(String srcHeaderId) {
		this.srcHeaderId = srcHeaderId;
	}

	public String getSrcLineId() {
		return srcLineId;
	}

	public void setSrcLineId(String srcLineId) {
		this.srcLineId = srcLineId;
	}

	public String getAsnHeaderId() {
		return asnHeaderId;
	}

	public void setAsnHeaderId(String asnHeaderId) {
		this.asnHeaderId = asnHeaderId;
	}

	public String getAsnLineId() {
		return asnLineId;
	}

	public void setAsnLineId(String asnLineId) {
		this.asnLineId = asnLineId;
	}

}
