package com.prolog.cs.biz.inbound.dto.failure;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 异常日志查询返回对象
 * 
 * @author Xiong
 *
 */
@ApiModel
public class ExceptionLogQueryDTO implements Serializable {

	private static final long serialVersionUID = -8014162012242130689L;

	@ApiModelProperty(value = "错误行日志ID(主键雪花id)(隐藏字段)", position = 1)
	private String exceptionLogId;

	@ApiModelProperty(value = "错误行ID(隐藏字段)", position = 1)
	private String exceptionLineId;

	@ApiModelProperty(value = "错误单头ID(隐藏字段)", position = 1)
	private String exceptionHeaderId;

	@ApiModelProperty(value = "收货行ID(隐藏字段)", position = 1)
	private String rcvLineId;

	@ApiModelProperty(value = "收货头ID(隐藏字段)", position = 1)
	private String rcvHeaderId;

	@ApiModelProperty(value = "行号")
	private BigDecimal lineNum;

	@ApiModelProperty(value = "货主ID(隐藏字段)", position = 1)
	private String mtlOwnerId;

	@ApiModelProperty(value = "仓库ID(隐藏字段)", position = 1)
	private String warehouseId;

	@ApiModelProperty(value = "商品ID(隐藏字段)", position = 1)
	private String itemId;

	@ApiModelProperty(value = "旧批次ID")
	private String batchIdOld;

	@ApiModelProperty(value = "旧批次编码")
	private String batchNumOld;

	@ApiModelProperty(value = "旧生产日期")
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date productionDateOld;

	@ApiModelProperty(value = "旧有效期至")
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date expiredDateOld;

	@ApiModelProperty(value = "旧数量")
	private BigDecimal qtyOld;

	@ApiModelProperty(value = "包装单位")
	private String uom;

	@ApiModelProperty(value = "包装含量")
	private BigDecimal conversionRate;

	@ApiModelProperty(value = "错误发现节点（10收货错误/20质检错误/30复检错误/40上架错误）")
	private String nodeType;

	@ApiModelProperty(value = "错误节点的任务id(隐藏字段)", position = 1)
	private String nodeTaskId;

	@ApiModelProperty(value = "旧供应商批次号")
	private String vendorBatchNumOld;

	@ApiModelProperty(value = "旧生产商批号")
	private String manufacturerLotNumOld;

	@ApiModelProperty(value = "容器编码")
	private String containerCode;

	@ApiModelProperty(value = "货格编码")
	private String goodsCaseCode;

	@ApiModelProperty(value = "新批次id号")
	private String batchIdNew;

	@ApiModelProperty(value = "新批次编码")
	private String batchNumNew;

	@ApiModelProperty(value = "新生产日期")
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date productionDateNew;

	@ApiModelProperty(value = "新有效期至")
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date expiredDateNew;

	@ApiModelProperty(value = "新供应商批次号")
	private String vendorBatchNumNew;

	@ApiModelProperty(value = "新生产商批号")
	private String manufacturerLotNumNew;

	@ApiModelProperty(value = "新数量")
	private BigDecimal qtyNew;

	@ApiModelProperty(value = "包装单位类型(2019/2020)新老包装")
	private String uomType;

	@ApiModelProperty(value = "商品条码(列表展示用)", position = 2)
	private String itemBarcode;

	@ApiModelProperty(value = "旧数量(最小单位)")
	private BigDecimal pieceQtyOld;

	@ApiModelProperty(value = "新数量（最小单位）")
	private BigDecimal pieceQtyNew;

	@ApiModelProperty(value = "执行者（异常处理人）")
	private String executorUser;

	@ApiModelProperty(value = "操作时间")
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date executorDate;

	@ApiModelProperty(value = "来源单类型(隐藏字段)", position = 1)
	private String srcBillType;

	@ApiModelProperty(value = "来源单头ID(PO/XT/YC)(隐藏字段)", position = 1)
	private String srcHeaderId;

	@ApiModelProperty(value = "来源单ID(隐藏字段)", position = 1)
	private String srcLineId;

	@ApiModelProperty(value = "ASN行ID(隐藏字段)", position = 1)
	private String asnHeaderId;

	@ApiModelProperty(value = "ASN行ID(隐藏字段)", position = 1)
	private String asnLineId;

	public String getItemBarcode() {
		return itemBarcode;
	}

	public void setItemBarcode(String itemBarcode) {
		this.itemBarcode = itemBarcode;
	}

	public String getExceptionLogId() {
		return exceptionLogId;
	}

	public void setExceptionLogId(String exceptionLogId) {
		this.exceptionLogId = exceptionLogId;
	}

	public String getExceptionLineId() {
		return exceptionLineId;
	}

	public void setExceptionLineId(String exceptionLineId) {
		this.exceptionLineId = exceptionLineId;
	}

	public String getExceptionHeaderId() {
		return exceptionHeaderId;
	}

	public void setExceptionHeaderId(String exceptionHeaderId) {
		this.exceptionHeaderId = exceptionHeaderId;
	}

	public String getRcvLineId() {
		return rcvLineId;
	}

	public void setRcvLineId(String rcvLineId) {
		this.rcvLineId = rcvLineId;
	}

	public String getRcvHeaderId() {
		return rcvHeaderId;
	}

	public void setRcvHeaderId(String rcvHeaderId) {
		this.rcvHeaderId = rcvHeaderId;
	}

	public BigDecimal getLineNum() {
		return lineNum;
	}

	public void setLineNum(BigDecimal lineNum) {
		this.lineNum = lineNum;
	}

	public String getMtlOwnerId() {
		return mtlOwnerId;
	}

	public void setMtlOwnerId(String mtlOwnerId) {
		this.mtlOwnerId = mtlOwnerId;
	}

	public String getWarehouseId() {
		return warehouseId;
	}

	public void setWarehouseId(String warehouseId) {
		this.warehouseId = warehouseId;
	}

	public String getItemId() {
		return itemId;
	}

	public void setItemId(String itemId) {
		this.itemId = itemId;
	}

	public String getBatchIdOld() {
		return batchIdOld;
	}

	public void setBatchIdOld(String batchIdOld) {
		this.batchIdOld = batchIdOld;
	}

	public String getBatchNumOld() {
		return batchNumOld;
	}

	public void setBatchNumOld(String batchNumOld) {
		this.batchNumOld = batchNumOld;
	}

	public Date getProductionDateOld() {
		return productionDateOld;
	}

	public void setProductionDateOld(Date productionDateOld) {
		this.productionDateOld = productionDateOld;
	}

	public Date getExpiredDateOld() {
		return expiredDateOld;
	}

	public void setExpiredDateOld(Date expiredDateOld) {
		this.expiredDateOld = expiredDateOld;
	}

	public BigDecimal getQtyOld() {
		return qtyOld;
	}

	public void setQtyOld(BigDecimal qtyOld) {
		this.qtyOld = qtyOld;
	}

	public String getUom() {
		return uom;
	}

	public void setUom(String uom) {
		this.uom = uom;
	}

	public BigDecimal getConversionRate() {
		return conversionRate;
	}

	public void setConversionRate(BigDecimal conversionRate) {
		this.conversionRate = conversionRate;
	}

	public String getNodeType() {
		return nodeType;
	}

	public void setNodeType(String nodeType) {
		this.nodeType = nodeType;
	}

	public String getNodeTaskId() {
		return nodeTaskId;
	}

	public void setNodeTaskId(String nodeTaskId) {
		this.nodeTaskId = nodeTaskId;
	}

	public String getVendorBatchNumOld() {
		return vendorBatchNumOld;
	}

	public void setVendorBatchNumOld(String vendorBatchNumOld) {
		this.vendorBatchNumOld = vendorBatchNumOld;
	}

	public String getManufacturerLotNumOld() {
		return manufacturerLotNumOld;
	}

	public void setManufacturerLotNumOld(String manufacturerLotNumOld) {
		this.manufacturerLotNumOld = manufacturerLotNumOld;
	}

	public String getContainerCode() {
		return containerCode;
	}

	public void setContainerCode(String containerCode) {
		this.containerCode = containerCode;
	}

	public String getGoodsCaseCode() {
		return goodsCaseCode;
	}

	public void setGoodsCaseCode(String goodsCaseCode) {
		this.goodsCaseCode = goodsCaseCode;
	}

	public String getBatchIdNew() {
		return batchIdNew;
	}

	public void setBatchIdNew(String batchIdNew) {
		this.batchIdNew = batchIdNew;
	}

	public String getBatchNumNew() {
		return batchNumNew;
	}

	public void setBatchNumNew(String batchNumNew) {
		this.batchNumNew = batchNumNew;
	}

	public Date getProductionDateNew() {
		return productionDateNew;
	}

	public void setProductionDateNew(Date productionDateNew) {
		this.productionDateNew = productionDateNew;
	}

	public Date getExpiredDateNew() {
		return expiredDateNew;
	}

	public void setExpiredDateNew(Date expiredDateNew) {
		this.expiredDateNew = expiredDateNew;
	}

	public String getVendorBatchNumNew() {
		return vendorBatchNumNew;
	}

	public void setVendorBatchNumNew(String vendorBatchNumNew) {
		this.vendorBatchNumNew = vendorBatchNumNew;
	}

	public String getManufacturerLotNumNew() {
		return manufacturerLotNumNew;
	}

	public void setManufacturerLotNumNew(String manufacturerLotNumNew) {
		this.manufacturerLotNumNew = manufacturerLotNumNew;
	}

	public BigDecimal getQtyNew() {
		return qtyNew;
	}

	public void setQtyNew(BigDecimal qtyNew) {
		this.qtyNew = qtyNew;
	}

	public String getUomType() {
		return uomType;
	}

	public void setUomType(String uomType) {
		this.uomType = uomType;
	}

	public BigDecimal getPieceQtyOld() {
		return pieceQtyOld;
	}

	public void setPieceQtyOld(BigDecimal pieceQtyOld) {
		this.pieceQtyOld = pieceQtyOld;
	}

	public BigDecimal getPieceQtyNew() {
		return pieceQtyNew;
	}

	public void setPieceQtyNew(BigDecimal pieceQtyNew) {
		this.pieceQtyNew = pieceQtyNew;
	}

	public String getExecutorUser() {
		return executorUser;
	}

	public void setExecutorUser(String executorUser) {
		this.executorUser = executorUser;
	}

	public Date getExecutorDate() {
		return executorDate;
	}

	public void setExecutorDate(Date executorDate) {
		this.executorDate = executorDate;
	}

	public String getSrcBillType() {
		return srcBillType;
	}

	public void setSrcBillType(String srcBillType) {
		this.srcBillType = srcBillType;
	}

	public String getSrcHeaderId() {
		return srcHeaderId;
	}

	public void setSrcHeaderId(String srcHeaderId) {
		this.srcHeaderId = srcHeaderId;
	}

	public String getSrcLineId() {
		return srcLineId;
	}

	public void setSrcLineId(String srcLineId) {
		this.srcLineId = srcLineId;
	}

	public String getAsnHeaderId() {
		return asnHeaderId;
	}

	public void setAsnHeaderId(String asnHeaderId) {
		this.asnHeaderId = asnHeaderId;
	}

	public String getAsnLineId() {
		return asnLineId;
	}

	public void setAsnLineId(String asnLineId) {
		this.asnLineId = asnLineId;
	}

}
