package com.prolog.cs.biz.inbound.dto.rqc;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.prolog.cs.biz.inbound.constant.ValidatorGroup.validAdd;
import com.prolog.cs.biz.inbound.constant.ValidatorGroup.validEdit;
import com.prolog.framework.core.annotation.Column;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * @Description 质检单明细 关联收货单，可以查询出所有信息
 * 
 *              (结论：合格/不合格/待处理) 合格/不合格 ：上架任务池 待处理：复检任务池
 * @Author Xiong
 * @Date 2021-04-02
 */

@ApiModel
public class RqcLinesDTO implements Serializable {
	private static final long serialVersionUID = 563826982236189844L;
	@ApiModelProperty(value = "唯一行号ID(主键雪花id)", required = true)
	@NotBlank(message = "唯一行号ID(主键雪花id) can not blank!", groups = validEdit.class)
	private String rqcLineId;

	@ApiModelProperty(value = "单据id", required = true)
	@NotBlank(message = "单据id can not blank!", groups = validAdd.class)
	private String rqcHeaderId;

	@ApiModelProperty(value = "单据code")
	private String rqcHeaderNo;

	@ApiModelProperty(value = "质检任务池ID（来源RQC_TASK）", required = true)
	@NotBlank(message = "质检任务池ID（来源RQC_TASK） can not blank!", groups = validAdd.class)
	private String taskRqcId;

	@ApiModelProperty(value = "收货单头ID")
	private String rcvHeaderId;

	@ApiModelProperty(value = "收货行头ID")
	private String rcvLineId;

	@ApiModelProperty(value = "货主ID（冗余，关联收货单）")
	private String mtlOwnerId;

	@ApiModelProperty(value = "仓库ID（冗余，关联收货单）", required = true)
	@NotBlank(message = "仓库ID（冗余，关联收货单） can not blank!", groups = validAdd.class)
	private String warehouseId;

	@ApiModelProperty(value = "来源单类型（冗余，关联收货单）")
	private String srcBillType;

	@ApiModelProperty(value = "来源单头ID（冗余，关联收货单）")
	private String srcHeaderId;

	@ApiModelProperty(value = "来源单行ID（冗余，关联收货单）")
	private String srcLineId;

	@ApiModelProperty(value = "ASN头ID（冗余，关联收货单）")
	private String asnHeaderId;

	@ApiModelProperty(value = "ASN行ID（冗余，关联收货单）")
	private String asnLineId;

	@ApiModelProperty(value = "商品ID", required = true)
	@NotBlank(message = "商品ID can not blank!", groups = validAdd.class)
	private String itemId;

	@ApiModelProperty(value = "容器编码（冗余，关联收货单行）")
	private String containerCode;

	@ApiModelProperty(value = "货格编码（冗余，关联收货单行）")
	private String goodsCaseCode;

	@ApiModelProperty(value = "批次ID", required = true)
	@NotBlank(message = "批次ID can not blank!", groups = validAdd.class)
	private String batchId;

	@ApiModelProperty(value = "包装单位", required = true)
	@NotBlank(message = "包装单位 can not blank!", groups = validAdd.class)
	private String uom;

	@ApiModelProperty(value = "包装单位类型(2019/2020)新老包装")
//	@NotBlank(message = "包装单位类型(2019/2020)新老包装 can not blank!", groups = validAdd.class)
	private String uomType;

	@ApiModelProperty(value = "包装含量", required = true)
	@NotNull(message = "包装含量 can not blank!", groups = validAdd.class)
	private BigDecimal conversionRate;

	@ApiModelProperty(value = "最小单位数量", required = true)
	@NotNull(message = "最小单位数量 can not blank!", groups = validAdd.class)
	private BigDecimal pieceQty;

	@ApiModelProperty(value = "质检整箱数", required = true)
	@NotNull(message = "质检整箱数 can not blank!", groups = validAdd.class)
	private BigDecimal oddCaseQty;

	@ApiModelProperty(value = "质检零散数", required = true)
	@NotNull(message = "质检零散数 can not blank!", groups = validAdd.class)
	private BigDecimal oddPieceQty;

	@ApiModelProperty(value = "抽样整箱数")
//	@NotNull(message = "抽样整箱数 can not blank!", groups = validAdd.class)
	private BigDecimal qcOddCaseQty;

	@ApiModelProperty(value = "抽样零散数")
	private BigDecimal qcOddPieceQty;

	@ApiModelProperty(value = "实际抽样整箱数")
	private BigDecimal qcOddCaseActQty;

	@ApiModelProperty(value = "实际抽样零散数")
	private BigDecimal qcOddPieceActQty;

	@ApiModelProperty(value = "行备注(可写异常信息)")
	private String lineRemark;

	@ApiModelProperty(value = "质检状态（初始/确认）")
	private String qcStatus1;

	@ApiModelProperty(value = "质检结论1（0合格/1不合格/2待处理/3拒绝）")
	private String qcResult1;

	@ApiModelProperty(value = "质检原因1")
	private String qcReason1;

	@ApiModelProperty(value = "质检等级1")
	private String qcGrade1;

	@ApiModelProperty(value = "质检员1")
	private String qcOpcode1;

	@ApiModelProperty(value = "质检时间1")
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date qcDate1;

	@ApiModelProperty(value = "质检状态2（初始/确认）")
	private String qcStatus2;

	@ApiModelProperty(value = "质检结论2（0合格/1不合格/2待处理/3拒绝）")
	private String qcResult2;

	@ApiModelProperty(value = "质检原因2")
	private String qcReason2;

	@ApiModelProperty(value = "质检等级2")
	private String qcGrade2;

	@ApiModelProperty(value = "质检员2")
	private String qcOpcode2;

	@ApiModelProperty(value = "质检状态2（初始/确认）")
	private String qcStatus3;

	@ApiModelProperty(value = "质检结论2（0合格/1不合格/2待处理/3拒绝）")
	private String qcResult3;

	@ApiModelProperty(value = "质检原因3")
	private String qcReason3;

	@ApiModelProperty(value = "质检等级3")
	private String qcGrade3;

	@ApiModelProperty(value = "质检员3")
	private String qcOpcode3;

	@ApiModelProperty(value = "质检时间2")
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date qcDate2;

	@ApiModelProperty(value = "质检时间3")
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date qcDate3;

	@ApiModelProperty(value = "(质检结论状态)")
	private String stockStatus;

	@ApiModelProperty(value = "货品标志")
	private String itemFlag;

	@ApiModelProperty(value = "质检次数")
	private Integer times;

	@ApiModelProperty(value = "双人业务类型(0:正常 1：双人验收，默认正常)")
	private Integer itemHealthIsDouble;

	@ApiModelProperty(value = "商品条码(列表展示用)")
	private String itemBarcode;
	@ApiModelProperty(value = "商品体积(列表展示用)")
	private BigDecimal itemVolume;
	@ApiModelProperty(value = "商品重量(列表展示用)")
	private BigDecimal itemWeight;
	@ApiModelProperty(value = "质检是否双票(列表展示用)")
	private Integer itemHealthTwovoflag;
	@ApiModelProperty(value = "仓库名称(列表展示用)")
	private String warehouseName;
	@ApiModelProperty(value = "库区名称(列表展示用)")
	private String warehouseAreaName;
	@ApiModelProperty(value = "商品编码(列表展示用)")
	private String itemNum;
	@ApiModelProperty(value = "商品名称(列表展示用)")
	private String itemName;
	@ApiModelProperty(value = "商品分类名(列表展示用)")
	private String categoryName;

	@ApiModelProperty(value = "生产商批号")
	private String manufacturerLotNum;

	@ApiModelProperty(value = "批次号")
	private String batchNum;

	@ApiModelProperty(value = "批次生产日期(列表展示用)", position = 9)
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date batchProductionDate;
	@ApiModelProperty(value = "批次有效期(列表展示用)", position = 9)
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date batchExpiredDate;
	@ApiModelProperty(value = "商品分类code名(展示用)", position = 9)
	private String categoryId;
	@ApiModelProperty(value = "仓库编码", position = 9)
	private String warehouseCode;

	// -------------------库存服务所需字段--------------------------------
	// 供应商id
	@ApiModelProperty(value = "供应商id(隐藏字段)", position = 9)
	private String vendorId;
	// 组织id
	@ApiModelProperty(value = "组织id(隐藏字段)", position = 9)
	private String orgId;
	// erp
	@ApiModelProperty(value = "erp批号(隐藏字段)", position = 9)
	private String erpBatchNum;
	// 产地
	@ApiModelProperty(value = "商品产地(隐藏字段)", position = 9)
	private String productionPlace;
	// 单品含税额
	@ApiModelProperty(value = "单品含税进价(隐藏字段)", position = 9)
	private BigDecimal piecePriceTi;
	// 单品不含税额
	@ApiModelProperty(value = "单品不含税进价(隐藏字段)", position = 9)
	private BigDecimal piecePriceTe;
	// 税率
	@ApiModelProperty(value = "税率(隐藏字段)", position = 9)
	private BigDecimal taxRate;

	@ApiModelProperty(value = "库别编码", position = 9)
	private String whAreaCategoryCode;

	// =================== 5.12 新增=========================================
	@ApiModelProperty(value = "(新)库区编号(逗号分隔列表)", position = -1)
	private String warehouseAreaCodes;
	@ApiModelProperty(value = "(新)商品整散分离标识(0整散合一/1整散分开)", position = -1)
	private String itemIsPieceOrCase;
	@ApiModelProperty(value = "(新)推荐库别编号(逗号分隔列表)", position = -1)
	private String warehouseAreaCategorys;

	// ================新增字段5.12=================================
	@ApiModelProperty(value = "(新)实际库别", position = -1)
	private String whAreaCategory;

	@ApiModelProperty(value = "(新)实际库区编号(逗号分隔集合)", position = -1)
	private String whAreaCodes;

	@ApiModelProperty(value = "(新)收货类型", position = -1)
	private String rcvType;

	// ===============5.17 新加字段================
	@ApiModelProperty(value = "(新)商品规格说明(冗余)", position = -1)
	private String specification;
	@ApiModelProperty(value = "(新)上市许可持有人(冗余)", position = -1)
	private String itemHealthListedLicensor;
	@ApiModelProperty(value = "(新)生产企业许可证号(冗余)", position = -1)
	private String itemHealthProduceLicense;
	@ApiModelProperty(value = "(新)存储分类(冗余)", position = -1)
	private String itemHealthStoreCategory;
	@ApiModelProperty(value = "(新)商品是否进口(冗余)", position = -1)
	private Integer itemIsImported;
	@ApiModelProperty(value = "(新)商品是否复检(0否1是)(冗余)", position = -1)
	private Integer itemIsRecheck;
	@ApiModelProperty(value = "(新)商品是否器械(0否1是)(冗余)", position = -1)
	private Integer isInstrument;

	// 处理意见
	@ApiModelProperty(value = "(新)处理意见", position = -2)
	private String handlingSuggestion;
	// 不足原因
	@ApiModelProperty(value = "(新)不足原因", position = -2)
	private String defReason;
	// 抽样日期
	@ApiModelProperty(value = "抽样日期", position = -2)
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date sampleDate;

	// ========5.20新加==========================
	// 商品通用名
	@ApiModelProperty(value = "(新)商品通用名(冗余)", position = -2)
	private String itemHealthCommonName;
	// 商品是否锁定
	@ApiModelProperty(value = "(新)商品是否锁定(0否1是)(冗余)", position = -2)
	private Integer isLock;
	// 商品是否代管
	@ApiModelProperty(value = "(新)商品是否代管(0否1是)(冗余)", position = -2)
	private Integer isHosted;
	// 生产厂家id
	@ApiModelProperty(value = "(新)生产厂家id(冗余)", position = -2)
	private String manufacturerId;
	// 生产厂家code
	@ApiModelProperty(value = "(新)生产厂家code(冗余)", position = -2)
	private String manufacturerCode;
	// 生产厂家name
	@ApiModelProperty(value = "(新)生产厂家name(冗余)", position = -2)
	private String manufacturerName;
	@ApiModelProperty(value = "(新)入库预留保质天数(冗余)", position = -2)
	private BigDecimal inStoreDay;
	// 注册证号
	@ApiModelProperty(value = "(新)注册证号(冗余)", position = -2)
	private String itemHealthRegisterNbr;
	// 有效期天数(需转换)
	@ApiModelProperty(value = "(新)有效期天数(需转换)(冗余)", position = -2)
	private BigDecimal shelfLifeDays;
	// 保质期单位
	@ApiModelProperty(value = "(新)保质期单位(年/月/日,默认日)(冗余)", position = -2)
	private String shelfLifeUnit;
	// 是否特殊品
	@ApiModelProperty(value = "(新)商品是否特殊品(0否1是)(冗余)", position = -2)
	private Integer itemIsSpecial;
	// 是否无菌
	@ApiModelProperty(value = "(新)商品是否无菌(0否1是)(冗余)", position = -2)
	private Integer isBacteria;
	// 退货单位提供的商品售出期间储存、运输质量控制情况
	@ApiModelProperty(value = "(新)退货单位提供的商品售出期间储存、运输质量控制情况(冗余)", position = -2)
	private String returnInfo;
	// 退货冷藏品出库期质量
	@ApiModelProperty(value = "(新)退货冷藏品出库期质量(冗余)", position = -2)
	private String returnColdQuality;
	// 退货说明
	@ApiModelProperty(value = "(新)退货说明(冗余)", position = -2)
	private String returnExplain;

	@ApiModelProperty(value = "是否复检过(0否1是)", position = -2)
	private Integer isRechecked;

	@ApiModelProperty(value = "是否抽样过(0否1是)", position = -2)
	private Integer isSpotCheck;

	@ApiModelProperty(value = "(新)抽样id", position = -2)
	private String rqcSampleId;

	@ApiModelProperty(value = "(新)处理原因(码表)", position = -2)
	private String handlingReason;
	@ApiModelProperty(value = "(新)不足原因补充", position = -2)
	private String defReasonExt;
	@ApiModelProperty(value = "(新)处理措施(码表)", position = -2)
	private String handlingMeasure;

	// ===========6.2号新加字段=====================================================
	@ApiModelProperty(value = "(新)是否打印上架标签(0否1是)", position = -1109)
	private Integer isInstockPrint;

	// ==============6.4号新加字段(打印单据用)======================================================
	@ApiModelProperty(value = "(新)收货员", position = -1109)
	private String rcvUser;// 收货员
	@ApiModelProperty(value = "(新)供应商", position = -1109)
	private String vendorName;// 供应商
	@ApiModelProperty(value = "(新)供应商", position = -1109)
	private String vendorNum;// 供应商
	@ApiModelProperty(value = "(新)质检日期", position = -1109)
	private Date rqcDate;// 质检日期
	@ApiModelProperty(value = "(新)组织名", position = -1109)
	private String orgnName;// 组织名(rqc头表缺失)
	@ApiModelProperty(value = "(新)货主名", position = -1109)
	private String mtlOwnerName;// 货主名
	@ApiModelProperty(value = "(新)采购员", position = -1109)
	private String linkMan;// 采购员
	@ApiModelProperty(value = "(新)采购单号(asn单号)", position = -1109)
	private String poNum;// 采购单号

	@ApiModelProperty(value = "是否监管", position = -1109)
	private String isRetpospect;

	@ApiModelProperty(value = "剂型", position = -2000)
	private String itemHealthType;

	// ====================6.10新加字段=================================
	// 退货类型 -- 收货行拓展
	@ApiModelProperty(value = "(新)退货类型(码表rcv_return_type)", position = -1110)
	private String returnType;
	// 提货说明
	@ApiModelProperty(value = "(新)提货说明", position = -1110)
	private String pickGoodsExplain;

	// 来源行id
	@ApiModelProperty(value = "(新)入库来源行id", position = -1111)
	private String refLineId;

	// 来源类型
	@ApiModelProperty(value = "(新)入库来源类型(0收货1质检2复检)", position = -1111)
	private Integer refType;

	@ApiModelProperty(value = "(新)显示生产日期", position = -1112)
	private String showProductionDate;
	@ApiModelProperty(value = "(新)显示有效期至", position = -1112)
	private String showExpiredDate;

	@ApiModelProperty(value = "(新)商品助记码", position = -1113)
	private String itemZjm;
	@ApiModelProperty(value = "(新)商品批准文号", position = -1113)
	private String approvalNumber;

	@ApiModelProperty(value = "(新)受托生产企业名称", position = -4)
	private String delegationPartyName;
	@ApiModelProperty(value = "(新)是否受托生产(0否1是)", position = -4)
	private Integer isDelegate;

	@ApiModelProperty(value = "(新)质检报告单号(关联批次)", position = -1113)
	private String rqcReportNo;

	@ApiModelProperty(value = "(新)验收温度", position = -1110)
	private String rqcTemperature;

	@ApiModelProperty(value = "(新)账期类型", position = -4)
	private String saleBillType;

	@ApiModelProperty(value = "(新)收货+质检备注", position = -4)
	private String remarks;

	@ApiModelProperty(value = "小数拆分标识(Y为允许,其他均不可)", position = -2)
	private String allowDecimal;
	
	@ApiModelProperty(value = "二级库存(id&code)", position = -2000)
	private String secondStatus;

	@ApiModelProperty(value = "复检任务id", position = -2000)
	private String taskRecheckId;

	@ApiModelProperty(value = "是否监管码扫描(0否1是默认0)", position = -2)
	private Integer isSupervise;

	@ApiModelProperty(value = "是否解锁：0=否 1=是")
	private Integer unlock;

	@ApiModelProperty(value = "小包装数", position = -3)
	private String minSu;

	@ApiModelProperty(value = "中包装数", position = -3)
	private String middleSu;

	@ApiModelProperty(value = "大包装数", position = -3)
	private String maxSu;

	@ApiModelProperty(value = "验收单号", position = -3)
	private String rqcNo;

	@ApiModelProperty(value = "供应商电话", position = -4)
	private String vendorPhone;


	@ApiModelProperty(value = "验收员", position = -4)
	private String rqcUser;

	@ApiModelProperty(value = "库别集合", position = -4)
	private List<String> whAreaCategoryCodeList;

	@ApiModelProperty(value = "存储条件", position = -4)
	private String storeTemperature;

	@ApiModelProperty(value = "是否能入AGVK", position = -2018)
	private String isAgv;

	public String getRqcUser() {
		return rqcUser;
	}

	public void setRqcUser(String rqcUser) {
		this.rqcUser = rqcUser;
	}

	public String getRqcNo() {
		return rqcNo;
	}

	public void setRqcNo(String rqcNo) {
		this.rqcNo = rqcNo;
	}

	public String getVendorPhone() {
		return vendorPhone;
	}

	public void setVendorPhone(String vendorPhone) {
		this.vendorPhone = vendorPhone;
	}

	public String getMinSu() {
		return minSu;
	}

	public void setMinSu(String minSu) {
		this.minSu = minSu;
	}

	public String getMiddleSu() {
		return middleSu;
	}

	public void setMiddleSu(String middleSu) {
		this.middleSu = middleSu;
	}

	public String getMaxSu() {
		return maxSu;
	}

	public void setMaxSu(String maxSu) {
		this.maxSu = maxSu;
	}

	public Integer getIsSupervise() {
		return isSupervise;
	}

	public void setIsSupervise(Integer isSupervise) {
		this.isSupervise = isSupervise;
	}

	public Integer getUnlock() {
		return unlock;
	}

	public void setUnlock(Integer unlock) {
		this.unlock = unlock;
	}

	public String getTaskRecheckId() { return taskRecheckId; }

	public void setTaskRecheckId(String taskRecheckId) { this.taskRecheckId = taskRecheckId; }

	public String getSecondStatus() {
		return secondStatus;
	}

	public void setSecondStatus(String secondStatus) {
		this.secondStatus = secondStatus;
	}

	@ApiModelProperty(value = "单位名称", position = -2)
	private String shipperName;

//	@ApiModelProperty(value = "小数拆分标识(Y为允许,其他均不可)", position = -2)
//	private String whAreaCategoryCode;
	@ApiModelProperty(value = "上架任务推荐货位id", position = -2)
	private String whLocatorId;

	@ApiModelProperty(value = "上架任务推荐货位编码", position = -2)
	private String whLocatorCode;

	@ApiModelProperty(value = "上架任务推荐货位名称", position = -2)
	private String whLocatorName;

	@ApiModelProperty(value = "上架任务建筑物编码", position = -2)
	private String bldgCode;

	@ApiModelProperty(value = "上架任务推荐库区编码", position = -2)
	private String whAreaCode;

	@ApiModelProperty(value = "上架任务标签", position = -2)
	private String instockLabel;

	@ApiModelProperty(value = "上架任务推荐货位楼层", position = -2)
	private String locatorFloorNum;

	@ApiModelProperty(value = "退货原因")
	private String rcvRtReason;

	@ApiModelProperty(value = "源容器号")
	private String oldContainerCode;

	@ApiModelProperty(value = "单据编号")
	private String rcvNo;

	@ApiModelProperty(value = "收货日期")
	private Date rcvDate;

	@ApiModelProperty(value = "单据类型")
	private String billTypeName;

	@ApiModelProperty(value = "上传监管码状态 0 空(不需要上传) 1 成功 2 失败")
	private Integer uploadCircubillStatus;

	@ApiModelProperty(value = "上传次数")
	private Integer uploadTimes;

	@ApiModelProperty(value = "最后上传时间")
	private Date lastUploadTime;

	public String getRcvRtReason() {
		return rcvRtReason;
	}

	public void setRcvRtReason(String rcvRtReason) {
		this.rcvRtReason = rcvRtReason;
	}

	public String getShipperName() {
		return shipperName;
	}

	public void setShipperName(String shipperName) {
		this.shipperName = shipperName;
	}

	public String getWhLocatorId() {
		return whLocatorId;
	}

	public void setWhLocatorId(String whLocatorId) {
		this.whLocatorId = whLocatorId;
	}

	public String getWhLocatorCode() {
		return whLocatorCode;
	}

	public void setWhLocatorCode(String whLocatorCode) {
		this.whLocatorCode = whLocatorCode;
	}

	public String getWhLocatorName() {
		return whLocatorName;
	}

	public void setWhLocatorName(String whLocatorName) {
		this.whLocatorName = whLocatorName;
	}

	public String getBldgCode() {
		return bldgCode;
	}

	public void setBldgCode(String bldgCode) {
		this.bldgCode = bldgCode;
	}

	public String getWhAreaCode() {
		return whAreaCode;
	}

	public void setWhAreaCode(String whAreaCode) {
		this.whAreaCode = whAreaCode;
	}

	public String getInstockLabel() {
		return instockLabel;
	}

	public void setInstockLabel(String instockLabel) {
		this.instockLabel = instockLabel;
	}

	public String getLocatorFloorNum() {
		return locatorFloorNum;
	}

	public void setLocatorFloorNum(String locatorFloorNum) {
		this.locatorFloorNum = locatorFloorNum;
	}

	public String getAllowDecimal() {
		return allowDecimal;
	}

	public void setAllowDecimal(String allowDecimal) {
		this.allowDecimal = allowDecimal;
	}

	public String getRemarks() {
		return remarks;
	}

	public void setRemarks(String remarks) {
		this.remarks = remarks;
	}

	public String getSaleBillType() {
		return saleBillType;
	}

	public void setSaleBillType(String saleBillType) {
		this.saleBillType = saleBillType;
	}

	public String getRqcTemperature() {
		return rqcTemperature;
	}

	public void setRqcTemperature(String rqcTemperature) {
		this.rqcTemperature = rqcTemperature;
	}

	public String getRqcReportNo() {
		return rqcReportNo;
	}

	public void setRqcReportNo(String rqcReportNo) {
		this.rqcReportNo = rqcReportNo;
	}

	public String getRqcHeaderNo() {
		return rqcHeaderNo;
	}

	public void setRqcHeaderNo(String rqcHeaderNo) {
		this.rqcHeaderNo = rqcHeaderNo;
	}

	public String getDelegationPartyName() {
		return delegationPartyName;
	}

	public void setDelegationPartyName(String delegationPartyName) {
		this.delegationPartyName = delegationPartyName;
	}

	public Integer getIsDelegate() {
		return isDelegate;
	}

	public void setIsDelegate(Integer isDelegate) {
		this.isDelegate = isDelegate;
	}

	public String getApprovalNumber() {
		return approvalNumber;
	}

	public void setApprovalNumber(String approvalNumber) {
		this.approvalNumber = approvalNumber;
	}

	public String getItemZjm() {
		return itemZjm;
	}

	public void setItemZjm(String itemZjm) {
		this.itemZjm = itemZjm;
	}

	public String getShowProductionDate() {
		return showProductionDate;
	}

	public void setShowProductionDate(String showProductionDate) {
		this.showProductionDate = showProductionDate;
	}

	public String getShowExpiredDate() {
		return showExpiredDate;
	}

	public void setShowExpiredDate(String showExpiredDate) {
		this.showExpiredDate = showExpiredDate;
	}

	public Integer getRefType() {
		return refType;
	}

	public void setRefType(Integer refType) {
		this.refType = refType;
	}

	public String getRefLineId() {
		return refLineId;
	}

	public void setRefLineId(String refLineId) {
		this.refLineId = refLineId;
	}

	public String getReturnType() {
		return returnType;
	}

	public void setReturnType(String returnType) {
		this.returnType = returnType;
	}

	public String getPickGoodsExplain() {
		return pickGoodsExplain;
	}

	public void setPickGoodsExplain(String pickGoodsExplain) {
		this.pickGoodsExplain = pickGoodsExplain;
	}

	public static long getSerialversionuid() {
		return serialVersionUID;
	}

	public String getPoNum() {
		return poNum;
	}

	public void setPoNum(String poNum) {
		this.poNum = poNum;
	}

	public String getMtlOwnerName() {
		return mtlOwnerName;
	}

	public void setMtlOwnerName(String mtlOwnerName) {
		this.mtlOwnerName = mtlOwnerName;
	}

	public String getLinkMan() {
		return linkMan;
	}

	public void setLinkMan(String linkMan) {
		this.linkMan = linkMan;
	}

	public String getRcvUser() {
		return rcvUser;
	}

	public void setRcvUser(String rcvUser) {
		this.rcvUser = rcvUser;
	}

	public String getVendorName() {
		return vendorName;
	}

	public void setVendorName(String vendorName) {
		this.vendorName = vendorName;
	}

	public Date getRqcDate() {
		return rqcDate;
	}

	public void setRqcDate(Date rqcDate) {
		this.rqcDate = rqcDate;
	}

	public String getOrgnName() {
		return orgnName;
	}

	public void setOrgnName(String orgnName) {
		this.orgnName = orgnName;
	}

	public Integer getIsInstockPrint() {
		return isInstockPrint;
	}

	public void setIsInstockPrint(Integer isInstockPrint) {
		this.isInstockPrint = isInstockPrint;
	}

	public String getHandlingReason() {
		return handlingReason;
	}

	public void setHandlingReason(String handlingReason) {
		this.handlingReason = handlingReason;
	}

	public String getDefReasonExt() {
		return defReasonExt;
	}

	public void setDefReasonExt(String defReasonExt) {
		this.defReasonExt = defReasonExt;
	}

	public String getHandlingMeasure() {
		return handlingMeasure;
	}

	public void setHandlingMeasure(String handlingMeasure) {
		this.handlingMeasure = handlingMeasure;
	}

	public String getRqcSampleId() {
		return rqcSampleId;
	}

	public void setRqcSampleId(String rqcSampleId) {
		this.rqcSampleId = rqcSampleId;
	}

	public Integer getIsRechecked() {
		return isRechecked;
	}

	public void setIsRechecked(Integer isRechecked) {
		this.isRechecked = isRechecked;
	}

	public Integer getIsSpotCheck() {
		return isSpotCheck;
	}

	public void setIsSpotCheck(Integer isSpotCheck) {
		this.isSpotCheck = isSpotCheck;
	}

	public String getItemHealthCommonName() {
		return itemHealthCommonName;
	}

	public void setItemHealthCommonName(String itemHealthCommonName) {
		this.itemHealthCommonName = itemHealthCommonName;
	}

	public Integer getIsLock() {
		return isLock;
	}

	public void setIsLock(Integer isLock) {
		this.isLock = isLock;
	}

	public Integer getIsHosted() {
		return isHosted;
	}

	public void setIsHosted(Integer isHosted) {
		this.isHosted = isHosted;
	}

	public String getManufacturerId() {
		return manufacturerId;
	}

	public void setManufacturerId(String manufacturerId) {
		this.manufacturerId = manufacturerId;
	}

	public String getManufacturerCode() {
		return manufacturerCode;
	}

	public void setManufacturerCode(String manufacturerCode) {
		this.manufacturerCode = manufacturerCode;
	}

	public String getManufacturerName() {
		return manufacturerName;
	}

	public void setManufacturerName(String manufacturerName) {
		this.manufacturerName = manufacturerName;
	}

	public BigDecimal getInStoreDay() {
		return inStoreDay;
	}

	public void setInStoreDay(BigDecimal inStoreDay) {
		this.inStoreDay = inStoreDay;
	}

	public String getItemHealthRegisterNbr() {
		return itemHealthRegisterNbr;
	}

	public void setItemHealthRegisterNbr(String itemHealthRegisterNbr) {
		this.itemHealthRegisterNbr = itemHealthRegisterNbr;
	}

	public BigDecimal getShelfLifeDays() {
		return shelfLifeDays;
	}

	public void setShelfLifeDays(BigDecimal shelfLifeDays) {
		this.shelfLifeDays = shelfLifeDays;
	}

	public String getShelfLifeUnit() {
		return shelfLifeUnit;
	}

	public void setShelfLifeUnit(String shelfLifeUnit) {
		this.shelfLifeUnit = shelfLifeUnit;
	}

	public Integer getItemIsSpecial() {
		return itemIsSpecial;
	}

	public void setItemIsSpecial(Integer itemIsSpecial) {
		this.itemIsSpecial = itemIsSpecial;
	}

	public Integer getIsBacteria() {
		return isBacteria;
	}

	public void setIsBacteria(Integer isBacteria) {
		this.isBacteria = isBacteria;
	}

	public String getReturnInfo() {
		return returnInfo;
	}

	public void setReturnInfo(String returnInfo) {
		this.returnInfo = returnInfo;
	}

	public String getReturnColdQuality() {
		return returnColdQuality;
	}

	public void setReturnColdQuality(String returnColdQuality) {
		this.returnColdQuality = returnColdQuality;
	}

	public String getReturnExplain() {
		return returnExplain;
	}

	public void setReturnExplain(String returnExplain) {
		this.returnExplain = returnExplain;
	}

	public String getHandlingSuggestion() {
		return handlingSuggestion;
	}

	public void setHandlingSuggestion(String handlingSuggestion) {
		this.handlingSuggestion = handlingSuggestion;
	}

	public String getDefReason() {
		return defReason;
	}

	public void setDefReason(String defReason) {
		this.defReason = defReason;
	}

	public Date getSampleDate() {
		return sampleDate;
	}

	public void setSampleDate(Date sampleDate) {
		this.sampleDate = sampleDate;
	}

	public String getSpecification() {
		return specification;
	}

	public void setSpecification(String specification) {
		this.specification = specification;
	}

	public String getItemHealthListedLicensor() {
		return itemHealthListedLicensor;
	}

	public void setItemHealthListedLicensor(String itemHealthListedLicensor) {
		this.itemHealthListedLicensor = itemHealthListedLicensor;
	}

	public String getItemHealthProduceLicense() {
		return itemHealthProduceLicense;
	}

	public void setItemHealthProduceLicense(String itemHealthProduceLicense) {
		this.itemHealthProduceLicense = itemHealthProduceLicense;
	}

	public String getItemHealthStoreCategory() {
		return itemHealthStoreCategory;
	}

	public void setItemHealthStoreCategory(String itemHealthStoreCategory) {
		this.itemHealthStoreCategory = itemHealthStoreCategory;
	}

	public Integer getItemIsImported() {
		return itemIsImported;
	}

	public void setItemIsImported(Integer itemIsImported) {
		this.itemIsImported = itemIsImported;
	}

	public Integer getItemIsRecheck() {
		return itemIsRecheck;
	}

	public void setItemIsRecheck(Integer itemIsRecheck) {
		this.itemIsRecheck = itemIsRecheck;
	}

	public Integer getIsInstrument() {
		return isInstrument;
	}

	public void setIsInstrument(Integer isInstrument) {
		this.isInstrument = isInstrument;
	}

	public String getRcvType() {
		return rcvType;
	}

	public void setRcvType(String rcvType) {
		this.rcvType = rcvType;
	}

	public String getWhAreaCategory() {
		return whAreaCategory;
	}

	public void setWhAreaCategory(String whAreaCategory) {
		this.whAreaCategory = whAreaCategory;
	}

	public String getWhAreaCodes() {
		return whAreaCodes;
	}

	public void setWhAreaCodes(String whAreaCodes) {
		this.whAreaCodes = whAreaCodes;
	}

	public String getWarehouseAreaCodes() {
		return warehouseAreaCodes;
	}

	public void setWarehouseAreaCodes(String warehouseAreaCodes) {
		this.warehouseAreaCodes = warehouseAreaCodes;
	}

	public String getItemIsPieceOrCase() {
		return itemIsPieceOrCase;
	}

	public void setItemIsPieceOrCase(String itemIsPieceOrCase) {
		this.itemIsPieceOrCase = itemIsPieceOrCase;
	}

	public String getWarehouseAreaCategorys() {
		return warehouseAreaCategorys;
	}

	public void setWarehouseAreaCategorys(String warehouseAreaCategorys) {
		this.warehouseAreaCategorys = warehouseAreaCategorys;
	}

	public Date getBatchProductionDate() {
		return batchProductionDate;
	}

	public void setBatchProductionDate(Date batchProductionDate) {
		this.batchProductionDate = batchProductionDate;
	}

	public Date getBatchExpiredDate() {
		return batchExpiredDate;
	}

	public void setBatchExpiredDate(Date batchExpiredDate) {
		this.batchExpiredDate = batchExpiredDate;
	}

	public String getCategoryId() {
		return categoryId;
	}

	public void setCategoryId(String categoryId) {
		this.categoryId = categoryId;
	}

	public String getWarehouseCode() {
		return warehouseCode;
	}

	public void setWarehouseCode(String warehouseCode) {
		this.warehouseCode = warehouseCode;
	}

	public String getVendorId() {
		return vendorId;
	}

	public void setVendorId(String vendorId) {
		this.vendorId = vendorId;
	}

	public String getOrgId() {
		return orgId;
	}

	public void setOrgId(String orgId) {
		this.orgId = orgId;
	}

	public String getErpBatchNum() {
		return erpBatchNum;
	}

	public void setErpBatchNum(String erpBatchNum) {
		this.erpBatchNum = erpBatchNum;
	}

	public String getProductionPlace() {
		return productionPlace;
	}

	public void setProductionPlace(String productionPlace) {
		this.productionPlace = productionPlace;
	}

	public BigDecimal getPiecePriceTi() {
		return piecePriceTi;
	}

	public void setPiecePriceTi(BigDecimal piecePriceTi) {
		this.piecePriceTi = piecePriceTi;
	}

	public BigDecimal getPiecePriceTe() {
		return piecePriceTe;
	}

	public void setPiecePriceTe(BigDecimal piecePriceTe) {
		this.piecePriceTe = piecePriceTe;
	}

	public BigDecimal getTaxRate() {
		return taxRate;
	}

	public void setTaxRate(BigDecimal taxRate) {
		this.taxRate = taxRate;
	}

	public String getWhAreaCategoryCode() {
		return whAreaCategoryCode;
	}

	public void setWhAreaCategoryCode(String whAreaCategoryCode) {
		this.whAreaCategoryCode = whAreaCategoryCode;
	}

	public String getRqcLineId() {
		return rqcLineId;
	}

	public void setRqcLineId(String rqcLineId) {
		this.rqcLineId = rqcLineId;
	}

	public String getRqcHeaderId() {
		return rqcHeaderId;
	}

	public void setRqcHeaderId(String rqcHeaderId) {
		this.rqcHeaderId = rqcHeaderId;
	}

	public String getTaskRqcId() {
		return taskRqcId;
	}

	public void setTaskRqcId(String taskRqcId) {
		this.taskRqcId = taskRqcId;
	}

	public String getRcvHeaderId() {
		return rcvHeaderId;
	}

	public void setRcvHeaderId(String rcvHeaderId) {
		this.rcvHeaderId = rcvHeaderId;
	}

	public String getRcvLineId() {
		return rcvLineId;
	}

	public void setRcvLineId(String rcvLineId) {
		this.rcvLineId = rcvLineId;
	}

	public String getMtlOwnerId() {
		return mtlOwnerId;
	}

	public void setMtlOwnerId(String mtlOwnerId) {
		this.mtlOwnerId = mtlOwnerId;
	}

	public String getWarehouseId() {
		return warehouseId;
	}

	public void setWarehouseId(String warehouseId) {
		this.warehouseId = warehouseId;
	}

	public String getSrcBillType() {
		return srcBillType;
	}

	public void setSrcBillType(String srcBillType) {
		this.srcBillType = srcBillType;
	}

	public String getSrcHeaderId() {
		return srcHeaderId;
	}

	public void setSrcHeaderId(String srcHeaderId) {
		this.srcHeaderId = srcHeaderId;
	}

	public String getSrcLineId() {
		return srcLineId;
	}

	public void setSrcLineId(String srcLineId) {
		this.srcLineId = srcLineId;
	}

	public String getAsnHeaderId() {
		return asnHeaderId;
	}

	public void setAsnHeaderId(String asnHeaderId) {
		this.asnHeaderId = asnHeaderId;
	}

	public String getAsnLineId() {
		return asnLineId;
	}

	public void setAsnLineId(String asnLineId) {
		this.asnLineId = asnLineId;
	}

	public String getItemId() {
		return itemId;
	}

	public void setItemId(String itemId) {
		this.itemId = itemId;
	}

	public String getContainerCode() {
		return containerCode;
	}

	public void setContainerCode(String containerCode) {
		this.containerCode = containerCode;
	}

	public String getGoodsCaseCode() {
		return goodsCaseCode;
	}

	public void setGoodsCaseCode(String goodsCaseCode) {
		this.goodsCaseCode = goodsCaseCode;
	}

	public String getBatchId() {
		return batchId;
	}

	public void setBatchId(String batchId) {
		this.batchId = batchId;
	}

	public String getUom() {
		return uom;
	}

	public void setUom(String uom) {
		this.uom = uom;
	}

	public String getUomType() {
		return uomType;
	}

	public void setUomType(String uomType) {
		this.uomType = uomType;
	}

	public BigDecimal getConversionRate() {
		return conversionRate;
	}

	public void setConversionRate(BigDecimal conversionRate) {
		this.conversionRate = conversionRate;
	}

	public BigDecimal getPieceQty() {
		return pieceQty;
	}

	public void setPieceQty(BigDecimal pieceQty) {
		this.pieceQty = pieceQty;
	}

	public BigDecimal getOddCaseQty() {
		return oddCaseQty;
	}

	public void setOddCaseQty(BigDecimal oddCaseQty) {
		this.oddCaseQty = oddCaseQty;
	}

	public BigDecimal getOddPieceQty() {
		return oddPieceQty;
	}

	public void setOddPieceQty(BigDecimal oddPieceQty) {
		this.oddPieceQty = oddPieceQty;
	}

	public BigDecimal getQcOddCaseQty() {
		return qcOddCaseQty;
	}

	public void setQcOddCaseQty(BigDecimal qcOddCaseQty) {
		this.qcOddCaseQty = qcOddCaseQty;
	}

	public BigDecimal getQcOddPieceQty() {
		return qcOddPieceQty;
	}

	public void setQcOddPieceQty(BigDecimal qcOddPieceQty) {
		this.qcOddPieceQty = qcOddPieceQty;
	}

	public BigDecimal getQcOddCaseActQty() {
		return qcOddCaseActQty;
	}

	public void setQcOddCaseActQty(BigDecimal qcOddCaseActQty) {
		this.qcOddCaseActQty = qcOddCaseActQty;
	}

	public BigDecimal getQcOddPieceActQty() {
		return qcOddPieceActQty;
	}

	public void setQcOddPieceActQty(BigDecimal qcOddPieceActQty) {
		this.qcOddPieceActQty = qcOddPieceActQty;
	}

	public String getLineRemark() {
		return lineRemark;
	}

	public void setLineRemark(String lineRemark) {
		this.lineRemark = lineRemark;
	}

	public String getQcStatus1() {
		return qcStatus1;
	}

	public void setQcStatus1(String qcStatus1) {
		this.qcStatus1 = qcStatus1;
	}

	public String getQcResult1() {
		return qcResult1;
	}

	public void setQcResult1(String qcResult1) {
		this.qcResult1 = qcResult1;
	}

	public String getQcReason1() {
		return qcReason1;
	}

	public void setQcReason1(String qcReason1) {
		this.qcReason1 = qcReason1;
	}

	public String getQcGrade1() {
		return qcGrade1;
	}

	public void setQcGrade1(String qcGrade1) {
		this.qcGrade1 = qcGrade1;
	}

	public String getQcOpcode1() {
		return qcOpcode1;
	}

	public void setQcOpcode1(String qcOpcode1) {
		this.qcOpcode1 = qcOpcode1;
	}

	public Date getQcDate1() {
		return qcDate1;
	}

	public void setQcDate1(Date qcDate1) {
		this.qcDate1 = qcDate1;
	}

	public String getQcStatus2() {
		return qcStatus2;
	}

	public void setQcStatus2(String qcStatus2) {
		this.qcStatus2 = qcStatus2;
	}

	public String getQcResult2() {
		return qcResult2;
	}

	public void setQcResult2(String qcResult2) {
		this.qcResult2 = qcResult2;
	}

	public String getQcReason2() {
		return qcReason2;
	}

	public void setQcReason2(String qcReason2) {
		this.qcReason2 = qcReason2;
	}

	public String getQcGrade2() {
		return qcGrade2;
	}

	public void setQcGrade2(String qcGrade2) {
		this.qcGrade2 = qcGrade2;
	}

	public String getQcOpcode2() {
		return qcOpcode2;
	}

	public void setQcOpcode2(String qcOpcode2) {
		this.qcOpcode2 = qcOpcode2;
	}

	public String getQcStatus3() {
		return qcStatus3;
	}

	public void setQcStatus3(String qcStatus3) {
		this.qcStatus3 = qcStatus3;
	}

	public String getQcResult3() {
		return qcResult3;
	}

	public void setQcResult3(String qcResult3) {
		this.qcResult3 = qcResult3;
	}

	public String getQcReason3() {
		return qcReason3;
	}

	public void setQcReason3(String qcReason3) {
		this.qcReason3 = qcReason3;
	}

	public String getQcGrade3() {
		return qcGrade3;
	}

	public void setQcGrade3(String qcGrade3) {
		this.qcGrade3 = qcGrade3;
	}

	public String getQcOpcode3() {
		return qcOpcode3;
	}

	public void setQcOpcode3(String qcOpcode3) {
		this.qcOpcode3 = qcOpcode3;
	}

	public Date getQcDate2() {
		return qcDate2;
	}

	public void setQcDate2(Date qcDate2) {
		this.qcDate2 = qcDate2;
	}

	public Date getQcDate3() {
		return qcDate3;
	}

	public void setQcDate3(Date qcDate3) {
		this.qcDate3 = qcDate3;
	}

	public String getStockStatus() {
		return stockStatus;
	}

	public void setStockStatus(String stockStatus) {
		this.stockStatus = stockStatus;
	}

	public String getItemFlag() {
		return itemFlag;
	}

	public void setItemFlag(String itemFlag) {
		this.itemFlag = itemFlag;
	}

	public Integer getItemHealthIsDouble() {
		return itemHealthIsDouble;
	}

	public void setItemHealthIsDouble(Integer itemHealthIsDouble) {
		this.itemHealthIsDouble = itemHealthIsDouble;
	}

	public Integer getTimes() {
		return times;
	}

	public void setTimes(Integer times) {
		this.times = times;
	}

	public String getItemBarcode() {
		return itemBarcode;
	}

	public void setItemBarcode(String itemBarcode) {
		this.itemBarcode = itemBarcode;
	}

	public BigDecimal getItemVolume() {
		return itemVolume;
	}

	public void setItemVolume(BigDecimal itemVolume) {
		this.itemVolume = itemVolume;
	}

	public BigDecimal getItemWeight() {
		return itemWeight;
	}

	public void setItemWeight(BigDecimal itemWeight) {
		this.itemWeight = itemWeight;
	}

	public Integer getItemHealthTwovoflag() {
		return itemHealthTwovoflag;
	}

	public void setItemHealthTwovoflag(Integer itemHealthTwovoflag) {
		this.itemHealthTwovoflag = itemHealthTwovoflag;
	}

	public String getWarehouseName() {
		return warehouseName;
	}

	public void setWarehouseName(String warehouseName) {
		this.warehouseName = warehouseName;
	}

	public String getWarehouseAreaName() {
		return warehouseAreaName;
	}

	public void setWarehouseAreaName(String warehouseAreaName) {
		this.warehouseAreaName = warehouseAreaName;
	}

	public String getItemNum() {
		return itemNum;
	}

	public void setItemNum(String itemNum) {
		this.itemNum = itemNum;
	}

	public String getItemName() {
		return itemName;
	}

	public void setItemName(String itemName) {
		this.itemName = itemName;
	}

	public String getCategoryName() {
		return categoryName;
	}

	public void setCategoryName(String categoryName) {
		this.categoryName = categoryName;
	}

	public String getManufacturerLotNum() {
		return manufacturerLotNum;
	}

	public void setManufacturerLotNum(String manufacturerLotNum) {
		this.manufacturerLotNum = manufacturerLotNum;
	}

	public String getBatchNum() {
		return batchNum;
	}

	public void setBatchNum(String batchNum) {
		this.batchNum = batchNum;
	}

	public List<String> getWhAreaCategoryCodeList() {
		return whAreaCategoryCodeList;
	}

	public void setWhAreaCategoryCodeList(List<String> whAreaCategoryCodeList) {
		this.whAreaCategoryCodeList = whAreaCategoryCodeList;
	}

	public String getStoreTemperature() {
		return storeTemperature;
	}

	public void setStoreTemperature(String storeTemperature) {
		this.storeTemperature = storeTemperature;
	}

	public String getIsAgv() {
		return isAgv;
	}

	public void setIsAgv(String isAgv) {
		this.isAgv = isAgv;
	}

	public String getOldContainerCode() {
		return oldContainerCode;
	}

	public void setOldContainerCode(String oldContainerCode) {
		this.oldContainerCode = oldContainerCode;
	}

	public String getRcvNo() {
		return rcvNo;
	}

	public void setRcvNo(String rcvNo) {
		this.rcvNo = rcvNo;
	}

	public Date getRcvDate() {
		return rcvDate;
	}

	public void setRcvDate(Date rcvDate) {
		this.rcvDate = rcvDate;
	}

	public String getBillTypeName() {
		return billTypeName;
	}

	public void setBillTypeName(String billTypeName) {
		this.billTypeName = billTypeName;
	}

	public String getVendorNum() {
		return vendorNum;
	}

	public void setVendorNum(String vendorNum) {
		this.vendorNum = vendorNum;
	}

	public String getIsRetpospect() {
		return isRetpospect;
	}

	public void setIsRetpospect(String isRetpospect) {
		this.isRetpospect = isRetpospect;
	}

	public String getItemHealthType() {
		return itemHealthType;
	}

	public void setItemHealthType(String itemHealthType) {
		this.itemHealthType = itemHealthType;
	}

	public Integer getUploadCircubillStatus() {
		return uploadCircubillStatus;
	}

	public void setUploadCircubillStatus(Integer uploadCircubillStatus) {
		this.uploadCircubillStatus = uploadCircubillStatus;
	}

	public Integer getUploadTimes() {
		return uploadTimes;
	}

	public void setUploadTimes(Integer uploadTimes) {
		this.uploadTimes = uploadTimes;
	}

	public Date getLastUploadTime() {
		return lastUploadTime;
	}

	public void setLastUploadTime(Date lastUploadTime) {
		this.lastUploadTime = lastUploadTime;
	}
}
