package com.prolog.cs.biz.inbound.dto.rqc;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.prolog.cs.biz.inbound.constant.ValidatorGroup.validAdd;
import com.prolog.cs.biz.inbound.constant.ValidatorGroup.validEdit;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * @Description 质检单明细 关联收货单，可以查询出所有信息
 * 
 *              (结论：合格/不合格/待处理) 合格/不合格 ：上架任务池 待处理：复检任务池
 * @Author Xiong
 * @Date 2021-04-02
 */

@ApiModel
public class RqcReLinesDTO implements Serializable {
	private static final long serialVersionUID = 2940408955134513850L;
	@ApiModelProperty(value = "唯一行号ID(主键雪花id)", required = true)
	@NotBlank(message = "唯一行号ID(主键雪花id) can not blank!", groups = validEdit.class)
	private String rqcReLineId;

	@ApiModelProperty(value = "单据id", required = true)
	@NotBlank(message = "单据id can not blank!", groups = validAdd.class)
	private String rqcReHeaderId;

	@ApiModelProperty(value = "质检任务池ID（来源RQC_RE_TASK）", required = true)
	@NotBlank(message = "质检任务池ID（来源RQC_RE_TASK） can not blank!", groups = validAdd.class)
	private String taskReRqcId;

	@ApiModelProperty(value = "收货单头ID")
	private String rcvHeaderId;

	@ApiModelProperty(value = "收货行头ID")
	private String rcvLineId;

	@ApiModelProperty(value = "货主ID（冗余，关联收货单）")
	private String mtlOwnerId;

	@ApiModelProperty(value = "仓库ID（冗余，关联收货单）", required = true)
	@NotBlank(message = "仓库ID（冗余，关联收货单） can not blank!", groups = validAdd.class)
	private String warehouseId;

	@ApiModelProperty(value = "来源单类型（冗余，关联收货单）")
	private String srcBillType;

	@ApiModelProperty(value = "来源单头ID（冗余，关联收货单）")
	private String srcHeaderId;

	@ApiModelProperty(value = "来源单行ID（冗余，关联收货单）")
	private String srcLineId;

	@ApiModelProperty(value = "ASN头ID（冗余，关联收货单）")
	private String asnHeaderId;

	@ApiModelProperty(value = "ASN行ID（冗余，关联收货单）")
	private String asnLineId;

	@ApiModelProperty(value = "商品ID", required = true)
	@NotBlank(message = "商品ID can not blank!", groups = validAdd.class)
	private String itemId;

	@ApiModelProperty(value = "容器编码（冗余，关联收货单行）")
	private String containerCode;

	@ApiModelProperty(value = "货格编码（冗余，关联收货单行）")
	private String goodsCaseCode;

	@ApiModelProperty(value = "批次ID")
//	@NotBlank(message = "批次ID can not blank!", groups = validAdd.class)
	private String batchId;

	@ApiModelProperty(value = "包装单位", required = true)
	@NotBlank(message = "包装单位 can not blank!", groups = validAdd.class)
	private String uom;

	@ApiModelProperty(value = "包装单位类型(2019/2020)新老包装")
	private String uomType;

	@ApiModelProperty(value = "包装含量", required = true)
	@NotNull(message = "包装含量 can not blank!", groups = validAdd.class)
	private BigDecimal conversionRate;

	@ApiModelProperty(value = "最小单位数量", required = true)
	@NotNull(message = "最小单位数量 can not blank!", groups = validAdd.class)
	private BigDecimal pieceQty;

	@ApiModelProperty(value = "收货整箱数", required = true)
	@NotNull(message = "收货整箱数 can not blank!", groups = validAdd.class)
	private BigDecimal oddCaseQty;

	@ApiModelProperty(value = "收货零散数", required = true)
	@NotNull(message = "收货零散数 can not blank!", groups = validAdd.class)
	private BigDecimal oddPieceQty;

	@ApiModelProperty(value = "抽样整箱数")
//	@NotNull(message = "抽样整箱数 can not blank!", groups = validAdd.class)
	private BigDecimal qcOddCaseQty;

	@ApiModelProperty(value = "抽样零散数")
//	@NotNull(message = "抽样零散数 can not blank!", groups = validAdd.class)
	private BigDecimal qcOddPieceQty;

	@ApiModelProperty(value = "实际抽样整箱数")
//	@NotNull(message = "实际抽样整箱数 can not blank!", groups = validAdd.class)
	private BigDecimal qcOddCaseActQty;

	@ApiModelProperty(value = "实际抽样零散数")
//	@NotNull(message = "实际抽样零散数 can not blank!", groups = validAdd.class)
	private BigDecimal qcOddPieceActQty;

	@ApiModelProperty(value = "行备注")
	private String lineRemark;

	@ApiModelProperty(value = "质检状态（初始/确认）")
//	@NotBlank(message = "质检状态（初始/确认） can not blank!", groups = validAdd.class)
	private String qcStatus;

	@ApiModelProperty(value = "质检结论（0合格/1不合格/2待处理/3拒绝）")
	private String qcResult;

	@ApiModelProperty(value = "质检原因")
	private String qcReason;

	@ApiModelProperty(value = "质检等级")
	private String qcGrade;

	@ApiModelProperty(value = "质检员")
	private String qcOpcode;

	@ApiModelProperty(value = "质检时间")
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date qcDate;

	@ApiModelProperty(value = "复检状态（初始/确认）")
	private String qcReStatus;

	@ApiModelProperty(value = "复检结论（0合格/1不合格/2待处理/3拒绝）", required = true, position = -10)
	private String qcReResult;

	@ApiModelProperty(value = "复检原因", required = true, position = -10)
	private String qcReReason;

	@ApiModelProperty(value = "复检结论0合格/1不合格/2待处理/3拒绝）", required = true, position = -10)
	private String stockStatus;

	@ApiModelProperty(value = "复检次数(从提取任务中带出来)", required = true, position = -10)
	private Integer times;

	@ApiModelProperty(value = "复检等级")
	private String qcReGrade;

	@ApiModelProperty(value = "复检员")
	private String qcReOpcode;

	@ApiModelProperty(value = "复检时间")
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date qcReDate;

	@ApiModelProperty(value = "货品标志")
	private String itemFlag;

	@ApiModelProperty(value = "商品条码(列表展示用)")
	private String itemBarcode;
	@ApiModelProperty(value = "商品体积(列表展示用)")
	private BigDecimal itemVolume;
	@ApiModelProperty(value = "商品重量(列表展示用)")
	private BigDecimal itemWeight;
	@ApiModelProperty(value = "质检是否双票(列表展示用)")
	private Integer itemHealthTwovoflag;
	@ApiModelProperty(value = "仓库名称(列表展示用)")
	private String warehouseName;
	@ApiModelProperty(value = "库区名称(列表展示用)")
	private String warehouseAreaName;
	@ApiModelProperty(value = "商品编码(列表展示用)")
	private String itemNum;
	@ApiModelProperty(value = "商品名称(列表展示用)")
	private String itemName;
	@ApiModelProperty(value = "商品分类名(列表展示用)")
	private String categoryName;
	@ApiModelProperty(value = "生产商批号")
	private String manufacturerLotNum;
	@ApiModelProperty(value = "批次号")
	private String batchNum;

	@ApiModelProperty(value = "批次生产日期(列表展示用)", position = 9)
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date batchProductionDate;
	@ApiModelProperty(value = "批次有效期(列表展示用)", position = 9)
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date batchExpiredDate;
	@ApiModelProperty(value = "商品分类code名(展示用)", position = 9)
	private String categoryId;
	@ApiModelProperty(value = "仓库编码", position = 9)
	private String warehouseCode;

	// =================== 5.12 新增=========================================
	@ApiModelProperty(value = "(新)库区编号(逗号分隔列表)", position = -1)
	private String warehouseAreaCodes;
	@ApiModelProperty(value = "(新)商品整散分离标识(0整散合一/1整散分开)", position = -1)
	private String itemIsPieceOrCase;
	@ApiModelProperty(value = "(新)推荐库别编号(逗号分隔列表)", position = -1)
	private String warehouseAreaCategorys;

	// ===============5.17 新加字段================
	@ApiModelProperty(value = "(新)收货类型", position = -1)
	private String rcvType;
	@ApiModelProperty(value = "(新)商品规格说明(冗余)", position = -1)
	private String specification;
	@ApiModelProperty(value = "(新)上市许可持有人(冗余)", position = -1)
	private String itemHealthListedLicensor;
	@ApiModelProperty(value = "(新)生产企业许可证号(冗余)", position = -1)
	private String itemHealthProduceLicense;
	@ApiModelProperty(value = "(新)存储分类(冗余)", position = -1)
	private String itemHealthStoreCategory;
	@ApiModelProperty(value = "(新)商品是否进口(冗余)", position = -1)
	private Integer itemIsImported;
	@ApiModelProperty(value = "(新)商品是否复检(0否1是)(冗余)", position = -1)
	private Integer itemIsRecheck;
	@ApiModelProperty(value = "(新)商品是否器械(0否1是)(冗余)", position = -1)
	private Integer isInstrument;

	// ========5.20新加==========================
	// 商品通用名
	@ApiModelProperty(value = "(新)商品通用名(冗余)", position = -2)
	private String itemHealthCommonName;
	// 商品是否锁定
	@ApiModelProperty(value = "(新)商品是否锁定(0否1是)(冗余)", position = -2)
	private Integer isLock;
	// 商品是否代管
	@ApiModelProperty(value = "(新)商品是否代管(0否1是)(冗余)", position = -2)
	private Integer isHosted;
	// 生产厂家id
	@ApiModelProperty(value = "(新)生产厂家id(冗余)", position = -2)
	private String manufacturerId;
	// 生产厂家code
	@ApiModelProperty(value = "(新)生产厂家id(冗余)", position = -2)
	private String manufacturerCode;
	// 生产厂家name
	@ApiModelProperty(value = "(新)生产厂家id(冗余)", position = -2)
	private String manufacturerName;
	@ApiModelProperty(value = "(新)入库预留保质天数(冗余)", position = -2)
	private BigDecimal inStoreDay;
	// 注册证号
	@ApiModelProperty(value = "(新)注册证号(冗余)", position = -2)
	private String itemHealthRegisterNbr;
	// 有效期天数(需转换)
	@ApiModelProperty(value = "(新)有效期天数(需转换)(冗余)", position = -2)
	private BigDecimal shelfLifeDays;
	// 保质期单位
	@ApiModelProperty(value = "(新)保质期单位(年/月/日,默认日)(冗余)", position = -2)
	private String shelfLifeUnit;
	// 是否特殊品
	@ApiModelProperty(value = "(新)商品是否特殊品(0否1是)(冗余)", position = -2)
	private Integer itemIsSpecial;
	// 是否无菌
	@ApiModelProperty(value = "(新)商品是否无菌(0否1是)(冗余)", position = -2)
	private Integer isBacteria;
	// 退货单位提供的商品售出期间储存、运输质量控制情况
	@ApiModelProperty(value = "(新)退货单位提供的商品售出期间储存、运输质量控制情况(冗余)", position = -2)
	private String returnInfo;
	// 退货冷藏品出库期质量
	@ApiModelProperty(value = "(新)退货冷藏品出库期质量(冗余)", position = -2)
	private String returnColdQuality;
	// 退货说明
	@ApiModelProperty(value = "(新)退货说明(冗余)", position = -2)
	private String returnExplain;

	// 来源行id
	@ApiModelProperty(value = "(新)入库来源行id", position = -5)
	private String refLineId;

	// 来源类型
	@ApiModelProperty(value = "(新)入库来源类型(0收货1质检2复检)", position = -5)
	private Integer refType;

	@ApiModelProperty(value = "处理意见", position = -6)
	private String handlingSuggestion;
	@ApiModelProperty(value = "审核意见", position = -6)
	private String aduitSuggestion;
	@ApiModelProperty(value = "供应商批号", position = -6)
	private String vendorBatchNum;

	@ApiModelProperty(value = "(新)显示生产日期", position = -1112)
	private String showProductionDate;
	@ApiModelProperty(value = "(新)显示有效期至", position = -1112)
	private String showExpiredDate;
	@ApiModelProperty(value = "(新)商品助记码", position = -1112)
	private String itemZjm;
	@ApiModelProperty(value = "(新)商品批准文号", position = -1113)
	private String approvalNumber;
	@ApiModelProperty(value = "商品产地(展示)", position = -1113)
	private String productionPlace;

	@ApiModelProperty(value = "复检意见1(主管)", position = -1114)
	private String qcReSuggestion;

	@ApiModelProperty(value = "复检结论2(经理)", position = -1114)
	private String qcReResult2;
	@ApiModelProperty(value = "复检人2(经理)", position = -1114)
	private String qcReOpcode2;
	@ApiModelProperty(value = "复检时间2(经理)", position = -1114)
	private Date qcReDate2;
	@ApiModelProperty(value = "复检意见2(经理)", position = -1114)
	private String qcReSuggestion2;

	@ApiModelProperty(value = "复检结论3(备用)", position = -1114)
	private String qcReResult3;
	@ApiModelProperty(value = "复检人3(备用)", position = -1114)
	private String qcReOpcode3;
	@ApiModelProperty(value = "复检时间3(备用)", position = -1114)
	private Date qcReDate3;
	@ApiModelProperty(value = "复检意见3(备用)", position = -1114)
	private String qcReSuggestion3;

	@ApiModelProperty(value = "双人业务类型(0:正常 1：双人验收，默认正常)")
	private Integer itemHealthIsDouble;

	@ApiModelProperty(value = "小数拆分标识(Y为允许,其他均不可)", position = -2)
	private String allowDecimal;

	@ApiModelProperty(value = "代管状态(来源是复检任务表ATTRIBUTE1字段)", position = -2000)
	private String hostedStatus;

	@ApiModelProperty(value = "质检单头ID")
	private String rqcHeaderId;

	public String getHostedStatus() {
		return hostedStatus;
	}

	public void setHostedStatus(String hostedStatus) {
		this.hostedStatus = hostedStatus;
	}

	public String getAllowDecimal() {
		return allowDecimal;
	}

	public void setAllowDecimal(String allowDecimal) {
		this.allowDecimal = allowDecimal;
	}

	public Integer getItemHealthIsDouble() {
		return itemHealthIsDouble;
	}

	public void setItemHealthIsDouble(Integer itemHealthIsDouble) {
		this.itemHealthIsDouble = itemHealthIsDouble;
	}

	public String getQcReSuggestion() {
		return qcReSuggestion;
	}

	public void setQcReSuggestion(String qcReSuggestion) {
		this.qcReSuggestion = qcReSuggestion;
	}

	public String getQcReResult2() {
		return qcReResult2;
	}

	public void setQcReResult2(String qcReResult2) {
		this.qcReResult2 = qcReResult2;
	}

	public String getQcReOpcode2() {
		return qcReOpcode2;
	}

	public void setQcReOpcode2(String qcReOpcode2) {
		this.qcReOpcode2 = qcReOpcode2;
	}

	public Date getQcReDate2() {
		return qcReDate2;
	}

	public void setQcReDate2(Date qcReDate2) {
		this.qcReDate2 = qcReDate2;
	}

	public String getQcReSuggestion2() {
		return qcReSuggestion2;
	}

	public void setQcReSuggestion2(String qcReSuggestion2) {
		this.qcReSuggestion2 = qcReSuggestion2;
	}

	public String getQcReResult3() {
		return qcReResult3;
	}

	public void setQcReResult3(String qcReResult3) {
		this.qcReResult3 = qcReResult3;
	}

	public String getQcReOpcode3() {
		return qcReOpcode3;
	}

	public void setQcReOpcode3(String qcReOpcode3) {
		this.qcReOpcode3 = qcReOpcode3;
	}

	public Date getQcReDate3() {
		return qcReDate3;
	}

	public void setQcReDate3(Date qcReDate3) {
		this.qcReDate3 = qcReDate3;
	}

	public String getQcReSuggestion3() {
		return qcReSuggestion3;
	}

	public void setQcReSuggestion3(String qcReSuggestion3) {
		this.qcReSuggestion3 = qcReSuggestion3;
	}

	public String getApprovalNumber() {
		return approvalNumber;
	}

	public void setApprovalNumber(String approvalNumber) {
		this.approvalNumber = approvalNumber;
	}

	public String getProductionPlace() {
		return productionPlace;
	}

	public void setProductionPlace(String productionPlace) {
		this.productionPlace = productionPlace;
	}

	public String getShowProductionDate() {
		return showProductionDate;
	}

	public void setShowProductionDate(String showProductionDate) {
		this.showProductionDate = showProductionDate;
	}

	public String getShowExpiredDate() {
		return showExpiredDate;
	}

	public void setShowExpiredDate(String showExpiredDate) {
		this.showExpiredDate = showExpiredDate;
	}

	public String getItemZjm() {
		return itemZjm;
	}

	public void setItemZjm(String itemZjm) {
		this.itemZjm = itemZjm;
	}

	public String getHandlingSuggestion() {
		return handlingSuggestion;
	}

	public void setHandlingSuggestion(String handlingSuggestion) {
		this.handlingSuggestion = handlingSuggestion;
	}

	public String getAduitSuggestion() {
		return aduitSuggestion;
	}

	public void setAduitSuggestion(String aduitSuggestion) {
		this.aduitSuggestion = aduitSuggestion;
	}

	public String getVendorBatchNum() {
		return vendorBatchNum;
	}

	public void setVendorBatchNum(String vendorBatchNum) {
		this.vendorBatchNum = vendorBatchNum;
	}

	public Integer getRefType() {
		return refType;
	}

	public void setRefType(Integer refType) {
		this.refType = refType;
	}

	public String getRefLineId() {
		return refLineId;
	}

	public void setRefLineId(String refLineId) {
		this.refLineId = refLineId;
	}

	public String getItemHealthCommonName() {
		return itemHealthCommonName;
	}

	public void setItemHealthCommonName(String itemHealthCommonName) {
		this.itemHealthCommonName = itemHealthCommonName;
	}

	public Integer getIsLock() {
		return isLock;
	}

	public void setIsLock(Integer isLock) {
		this.isLock = isLock;
	}

	public Integer getIsHosted() {
		return isHosted;
	}

	public void setIsHosted(Integer isHosted) {
		this.isHosted = isHosted;
	}

	public String getManufacturerId() {
		return manufacturerId;
	}

	public void setManufacturerId(String manufacturerId) {
		this.manufacturerId = manufacturerId;
	}

	public String getManufacturerCode() {
		return manufacturerCode;
	}

	public void setManufacturerCode(String manufacturerCode) {
		this.manufacturerCode = manufacturerCode;
	}

	public String getManufacturerName() {
		return manufacturerName;
	}

	public void setManufacturerName(String manufacturerName) {
		this.manufacturerName = manufacturerName;
	}

	public BigDecimal getInStoreDay() {
		return inStoreDay;
	}

	public void setInStoreDay(BigDecimal inStoreDay) {
		this.inStoreDay = inStoreDay;
	}

	public String getItemHealthRegisterNbr() {
		return itemHealthRegisterNbr;
	}

	public void setItemHealthRegisterNbr(String itemHealthRegisterNbr) {
		this.itemHealthRegisterNbr = itemHealthRegisterNbr;
	}

	public BigDecimal getShelfLifeDays() {
		return shelfLifeDays;
	}

	public void setShelfLifeDays(BigDecimal shelfLifeDays) {
		this.shelfLifeDays = shelfLifeDays;
	}

	public String getShelfLifeUnit() {
		return shelfLifeUnit;
	}

	public void setShelfLifeUnit(String shelfLifeUnit) {
		this.shelfLifeUnit = shelfLifeUnit;
	}

	public Integer getItemIsSpecial() {
		return itemIsSpecial;
	}

	public void setItemIsSpecial(Integer itemIsSpecial) {
		this.itemIsSpecial = itemIsSpecial;
	}

	public Integer getIsBacteria() {
		return isBacteria;
	}

	public void setIsBacteria(Integer isBacteria) {
		this.isBacteria = isBacteria;
	}

	public String getReturnInfo() {
		return returnInfo;
	}

	public void setReturnInfo(String returnInfo) {
		this.returnInfo = returnInfo;
	}

	public String getReturnColdQuality() {
		return returnColdQuality;
	}

	public void setReturnColdQuality(String returnColdQuality) {
		this.returnColdQuality = returnColdQuality;
	}

	public String getReturnExplain() {
		return returnExplain;
	}

	public void setReturnExplain(String returnExplain) {
		this.returnExplain = returnExplain;
	}

	public Date getBatchProductionDate() {
		return batchProductionDate;
	}

	public void setBatchProductionDate(Date batchProductionDate) {
		this.batchProductionDate = batchProductionDate;
	}

	public Date getBatchExpiredDate() {
		return batchExpiredDate;
	}

	public void setBatchExpiredDate(Date batchExpiredDate) {
		this.batchExpiredDate = batchExpiredDate;
	}

	public String getCategoryId() {
		return categoryId;
	}

	public void setCategoryId(String categoryId) {
		this.categoryId = categoryId;
	}

	public String getWarehouseCode() {
		return warehouseCode;
	}

	public void setWarehouseCode(String warehouseCode) {
		this.warehouseCode = warehouseCode;
	}

	public String getRcvType() {
		return rcvType;
	}

	public void setRcvType(String rcvType) {
		this.rcvType = rcvType;
	}

	public String getSpecification() {
		return specification;
	}

	public void setSpecification(String specification) {
		this.specification = specification;
	}

	public String getItemHealthListedLicensor() {
		return itemHealthListedLicensor;
	}

	public void setItemHealthListedLicensor(String itemHealthListedLicensor) {
		this.itemHealthListedLicensor = itemHealthListedLicensor;
	}

	public String getItemHealthProduceLicense() {
		return itemHealthProduceLicense;
	}

	public void setItemHealthProduceLicense(String itemHealthProduceLicense) {
		this.itemHealthProduceLicense = itemHealthProduceLicense;
	}

	public String getItemHealthStoreCategory() {
		return itemHealthStoreCategory;
	}

	public void setItemHealthStoreCategory(String itemHealthStoreCategory) {
		this.itemHealthStoreCategory = itemHealthStoreCategory;
	}

	public Integer getItemIsImported() {
		return itemIsImported;
	}

	public void setItemIsImported(Integer itemIsImported) {
		this.itemIsImported = itemIsImported;
	}

	public Integer getItemIsRecheck() {
		return itemIsRecheck;
	}

	public void setItemIsRecheck(Integer itemIsRecheck) {
		this.itemIsRecheck = itemIsRecheck;
	}

	public Integer getIsInstrument() {
		return isInstrument;
	}

	public void setIsInstrument(Integer isInstrument) {
		this.isInstrument = isInstrument;
	}

	public String getWarehouseAreaCodes() {
		return warehouseAreaCodes;
	}

	public void setWarehouseAreaCodes(String warehouseAreaCodes) {
		this.warehouseAreaCodes = warehouseAreaCodes;
	}

	public String getItemIsPieceOrCase() {
		return itemIsPieceOrCase;
	}

	public void setItemIsPieceOrCase(String itemIsPieceOrCase) {
		this.itemIsPieceOrCase = itemIsPieceOrCase;
	}

	public String getWarehouseAreaCategorys() {
		return warehouseAreaCategorys;
	}

	public void setWarehouseAreaCategorys(String warehouseAreaCategorys) {
		this.warehouseAreaCategorys = warehouseAreaCategorys;
	}

	public String getRqcReLineId() {
		return rqcReLineId;
	}

	public void setRqcReLineId(String rqcReLineId) {
		this.rqcReLineId = rqcReLineId;
	}

	public String getRqcReHeaderId() {
		return rqcReHeaderId;
	}

	public void setRqcReHeaderId(String rqcReHeaderId) {
		this.rqcReHeaderId = rqcReHeaderId;
	}

	public String getTaskReRqcId() {
		return taskReRqcId;
	}

	public void setTaskReRqcId(String taskReRqcId) {
		this.taskReRqcId = taskReRqcId;
	}

	public String getRcvHeaderId() {
		return rcvHeaderId;
	}

	public void setRcvHeaderId(String rcvHeaderId) {
		this.rcvHeaderId = rcvHeaderId;
	}

	public String getRcvLineId() {
		return rcvLineId;
	}

	public void setRcvLineId(String rcvLineId) {
		this.rcvLineId = rcvLineId;
	}

	public String getMtlOwnerId() {
		return mtlOwnerId;
	}

	public void setMtlOwnerId(String mtlOwnerId) {
		this.mtlOwnerId = mtlOwnerId;
	}

	public String getWarehouseId() {
		return warehouseId;
	}

	public void setWarehouseId(String warehouseId) {
		this.warehouseId = warehouseId;
	}

	public String getSrcBillType() {
		return srcBillType;
	}

	public void setSrcBillType(String srcBillType) {
		this.srcBillType = srcBillType;
	}

	public String getSrcHeaderId() {
		return srcHeaderId;
	}

	public void setSrcHeaderId(String srcHeaderId) {
		this.srcHeaderId = srcHeaderId;
	}

	public String getSrcLineId() {
		return srcLineId;
	}

	public void setSrcLineId(String srcLineId) {
		this.srcLineId = srcLineId;
	}

	public String getAsnHeaderId() {
		return asnHeaderId;
	}

	public void setAsnHeaderId(String asnHeaderId) {
		this.asnHeaderId = asnHeaderId;
	}

	public String getAsnLineId() {
		return asnLineId;
	}

	public void setAsnLineId(String asnLineId) {
		this.asnLineId = asnLineId;
	}

	public String getItemId() {
		return itemId;
	}

	public void setItemId(String itemId) {
		this.itemId = itemId;
	}

	public String getContainerCode() {
		return containerCode;
	}

	public void setContainerCode(String containerCode) {
		this.containerCode = containerCode;
	}

	public String getGoodsCaseCode() {
		return goodsCaseCode;
	}

	public void setGoodsCaseCode(String goodsCaseCode) {
		this.goodsCaseCode = goodsCaseCode;
	}

	public String getBatchId() {
		return batchId;
	}

	public void setBatchId(String batchId) {
		this.batchId = batchId;
	}

	public String getUom() {
		return uom;
	}

	public void setUom(String uom) {
		this.uom = uom;
	}

	public String getUomType() {
		return uomType;
	}

	public void setUomType(String uomType) {
		this.uomType = uomType;
	}

	public BigDecimal getConversionRate() {
		return conversionRate;
	}

	public void setConversionRate(BigDecimal conversionRate) {
		this.conversionRate = conversionRate;
	}

	public BigDecimal getPieceQty() {
		return pieceQty;
	}

	public void setPieceQty(BigDecimal pieceQty) {
		this.pieceQty = pieceQty;
	}

	public BigDecimal getOddCaseQty() {
		return oddCaseQty;
	}

	public void setOddCaseQty(BigDecimal oddCaseQty) {
		this.oddCaseQty = oddCaseQty;
	}

	public BigDecimal getOddPieceQty() {
		return oddPieceQty;
	}

	public void setOddPieceQty(BigDecimal oddPieceQty) {
		this.oddPieceQty = oddPieceQty;
	}

	public BigDecimal getQcOddCaseQty() {
		return qcOddCaseQty;
	}

	public void setQcOddCaseQty(BigDecimal qcOddCaseQty) {
		this.qcOddCaseQty = qcOddCaseQty;
	}

	public BigDecimal getQcOddPieceQty() {
		return qcOddPieceQty;
	}

	public void setQcOddPieceQty(BigDecimal qcOddPieceQty) {
		this.qcOddPieceQty = qcOddPieceQty;
	}

	public BigDecimal getQcOddCaseActQty() {
		return qcOddCaseActQty;
	}

	public void setQcOddCaseActQty(BigDecimal qcOddCaseActQty) {
		this.qcOddCaseActQty = qcOddCaseActQty;
	}

	public BigDecimal getQcOddPieceActQty() {
		return qcOddPieceActQty;
	}

	public void setQcOddPieceActQty(BigDecimal qcOddPieceActQty) {
		this.qcOddPieceActQty = qcOddPieceActQty;
	}

	public String getLineRemark() {
		return lineRemark;
	}

	public void setLineRemark(String lineRemark) {
		this.lineRemark = lineRemark;
	}

	public String getQcStatus() {
		return qcStatus;
	}

	public void setQcStatus(String qcStatus) {
		this.qcStatus = qcStatus;
	}

	public String getQcResult() {
		return qcResult;
	}

	public void setQcResult(String qcResult) {
		this.qcResult = qcResult;
	}

	public String getQcReason() {
		return qcReason;
	}

	public void setQcReason(String qcReason) {
		this.qcReason = qcReason;
	}

	public String getQcGrade() {
		return qcGrade;
	}

	public void setQcGrade(String qcGrade) {
		this.qcGrade = qcGrade;
	}

	public String getQcOpcode() {
		return qcOpcode;
	}

	public void setQcOpcode(String qcOpcode) {
		this.qcOpcode = qcOpcode;
	}

	public Date getQcDate() {
		return qcDate;
	}

	public void setQcDate(Date qcDate) {
		this.qcDate = qcDate;
	}

	public String getQcReStatus() {
		return qcReStatus;
	}

	public void setQcReStatus(String qcReStatus) {
		this.qcReStatus = qcReStatus;
	}

	public String getQcReResult() {
		return qcReResult;
	}

	public void setQcReResult(String qcReResult) {
		this.qcReResult = qcReResult;
	}

	public String getQcReReason() {
		return qcReReason;
	}

	public void setQcReReason(String qcReReason) {
		this.qcReReason = qcReReason;
	}

	public String getQcReGrade() {
		return qcReGrade;
	}

	public void setQcReGrade(String qcReGrade) {
		this.qcReGrade = qcReGrade;
	}

	public String getQcReOpcode() {
		return qcReOpcode;
	}

	public void setQcReOpcode(String qcReOpcode) {
		this.qcReOpcode = qcReOpcode;
	}

	public Date getQcReDate() {
		return qcReDate;
	}

	public void setQcReDate(Date qcReDate) {
		this.qcReDate = qcReDate;
	}

	public String getStockStatus() {
		return stockStatus;
	}

	public void setStockStatus(String stockStatus) {
		this.stockStatus = stockStatus;
	}

	public String getItemFlag() {
		return itemFlag;
	}

	public void setItemFlag(String itemFlag) {
		this.itemFlag = itemFlag;
	}

	public Integer getTimes() {
		return times;
	}

	public void setTimes(Integer times) {
		this.times = times;
	}

	public String getItemBarcode() {
		return itemBarcode;
	}

	public void setItemBarcode(String itemBarcode) {
		this.itemBarcode = itemBarcode;
	}

	public BigDecimal getItemVolume() {
		return itemVolume;
	}

	public void setItemVolume(BigDecimal itemVolume) {
		this.itemVolume = itemVolume;
	}

	public BigDecimal getItemWeight() {
		return itemWeight;
	}

	public void setItemWeight(BigDecimal itemWeight) {
		this.itemWeight = itemWeight;
	}

	public Integer getItemHealthTwovoflag() {
		return itemHealthTwovoflag;
	}

	public void setItemHealthTwovoflag(Integer itemHealthTwovoflag) {
		this.itemHealthTwovoflag = itemHealthTwovoflag;
	}

	public String getWarehouseName() {
		return warehouseName;
	}

	public void setWarehouseName(String warehouseName) {
		this.warehouseName = warehouseName;
	}

	public String getWarehouseAreaName() {
		return warehouseAreaName;
	}

	public void setWarehouseAreaName(String warehouseAreaName) {
		this.warehouseAreaName = warehouseAreaName;
	}

	public String getItemNum() {
		return itemNum;
	}

	public void setItemNum(String itemNum) {
		this.itemNum = itemNum;
	}

	public String getItemName() {
		return itemName;
	}

	public void setItemName(String itemName) {
		this.itemName = itemName;
	}

	public String getCategoryName() {
		return categoryName;
	}

	public void setCategoryName(String categoryName) {
		this.categoryName = categoryName;
	}

	public String getManufacturerLotNum() {
		return manufacturerLotNum;
	}

	public void setManufacturerLotNum(String manufacturerLotNum) {
		this.manufacturerLotNum = manufacturerLotNum;
	}

	public String getBatchNum() {
		return batchNum;
	}

	public void setBatchNum(String batchNum) {
		this.batchNum = batchNum;
	}

	public String getRqcHeaderId() {
		return rqcHeaderId;
	}

	public void setRqcHeaderId(String rqcHeaderId) {
		this.rqcHeaderId = rqcHeaderId;
	}
}
