package com.prolog.cs.biz.inbound.dto.rqc;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.prolog.cs.biz.inbound.constant.ValidatorGroup.validAdd;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Past;
import java.io.Serializable;
import java.util.Date;

/**
 * @Description 复检单头 
 * @Author Xiong  
 * @Date 2021-04-02 
 */

@ApiModel
public class RqcRechekHeadersDTO  implements Serializable {
	private static final long serialVersionUID =  4196307592340375444L;
	@ApiModelProperty(value = "业主编号(主键雪花id)",required = true) 
	@NotBlank(message = "业主编号(主键雪花id) can not blank!",groups = validAdd.class) 
	private String consignor;

	@ApiModelProperty(value = "采购单号",required = true) 
	@NotBlank(message = "采购单号 can not blank!",groups = validAdd.class) 
	private String originalbillno;

	@ApiModelProperty(value = "收货单号") 
	private String refbillno;

	@ApiModelProperty(value = "质检单号",required = true) 
	@NotBlank(message = "质检单号 can not blank!",groups = validAdd.class) 
	private String billno;

	@ApiModelProperty(value = "单据日期",required = true) 
	@NotNull(message = "单据日期 can not blank!",groups = validAdd.class) 
	@Past(message = "单据日期 can not greater than now!",groups = validAdd.class) 
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date billDate;

	@ApiModelProperty(value = "供应商编码",required = true) 
	@NotBlank(message = "供应商编码 can not blank!",groups = validAdd.class) 
	private String companycode;

	@ApiModelProperty(value = "质检状态(10初始11已打印13已确认)",required = true) 
	@NotBlank(message = "质检状态(10初始11已打印13已确认) can not blank!",groups = validAdd.class) 
	private String qcstatus;

	@ApiModelProperty(value = "复检状态") 
	private String reqcstatus;


	public String getConsignor() {
		return this.consignor;
	}

	public void setConsignor(String consignor) {
		this.consignor = consignor;
	}

	public String getOriginalbillno() {
		return this.originalbillno;
	}

	public void setOriginalbillno(String originalbillno) {
		this.originalbillno = originalbillno;
	}

	public String getRefbillno() {
		return this.refbillno;
	}

	public void setRefbillno(String refbillno) {
		this.refbillno = refbillno;
	}

	public String getBillno() {
		return this.billno;
	}

	public void setBillno(String billno) {
		this.billno = billno;
	}

	public Date getBillDate() {
		return this.billDate;
	}

	public void setBillDate(Date billDate) {
		this.billDate = billDate;
	}

	public String getCompanycode() {
		return this.companycode;
	}

	public void setCompanycode(String companycode) {
		this.companycode = companycode;
	}

	public String getQcstatus() {
		return this.qcstatus;
	}

	public void setQcstatus(String qcstatus) {
		this.qcstatus = qcstatus;
	}

	public String getReqcstatus() {
		return this.reqcstatus;
	}

	public void setReqcstatus(String reqcstatus) {
		this.reqcstatus = reqcstatus;
	}

}
