package com.prolog.cs.biz.inbound.dto.task;

import com.prolog.cs.biz.inbound.constant.ValidatorGroup.validAdd;
import com.prolog.cs.biz.inbound.constant.ValidatorGroup.validEdit;
import com.prolog.cs.biz.inbound.dto.IsSuperviseDTO;
import com.prolog.cs.biz.inbound.dto.rcv.RcvLinesDTO;
import com.prolog.cs.biz.inbound.dto.rcv.RcvLinesProfilesDTO;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * @Description 质检任务池
 * @Author Xiong
 * @Date 2021-04-02
 */

@ApiModel
public class TaskQcDetailRFDTO extends IsSuperviseDTO implements Serializable {

	private static final long serialVersionUID = 6160616960818856819L;

	@ApiModelProperty(value = "ID(主键雪花id)", required = true)
	@NotBlank(message = "ID(主键雪花id) can not blank!", groups = validEdit.class)
	private String taskRqcId;

	@ApiModelProperty(value = "货主ID")
	private String mtlOwnerId;

	@ApiModelProperty(value = "仓库ID", required = true)
	@NotBlank(message = "仓库ID can not blank!", groups = validAdd.class)
	private String warehouseId;

	@ApiModelProperty(value = "来源单类型")
	private String srcBillType;

	@ApiModelProperty(value = "来源单头ID")
	private String srcHeaderId;

	@ApiModelProperty(value = "来源单行ID")
	private String srcLineId;

	@ApiModelProperty(value = "ASN头ID")
	private String asnHeaderId;

	@ApiModelProperty(value = "ASN行ID")
	private String asnLineId;

	@ApiModelProperty(value = "收货单头ID")
	private String rcvHeaderId;

	@ApiModelProperty(value = "收货行头ID")
	private String rcvLineId;

	@ApiModelProperty(value = "商品ID")
	private String itemId;

	@ApiModelProperty(value = "包装数量(pieceQty/conversionRate)")
	private BigDecimal qty;

	@ApiModelProperty(value = "包装单位", required = true)
	@NotBlank(message = "包装单位 can not blank!", groups = validAdd.class)
	private String uom;

	@ApiModelProperty(value = "包装含量(件包装)", required = true)
	@NotNull(message = "包装含量 can not blank!", groups = validAdd.class)
	private BigDecimal conversionRate;

	@ApiModelProperty(value = "单品总数量", required = true)
	@NotNull(message = "单品数量 can not blank!", groups = validAdd.class)
	private BigDecimal pieceQty;

	@ApiModelProperty(value = "收货总数量", required = true)
	private BigDecimal receiveQty;

	@ApiModelProperty(value = "批次ID")
	private String batchId;

	@ApiModelProperty(value = "批次号")
	private String batchNum;

	@ApiModelProperty(value = "批号")
	private String vendorBatchNum;

	@ApiModelProperty(value = "是否允许作业标识")
	private String activeFlag;

	@ApiModelProperty(value = "余量")
	private BigDecimal remainQty;

	@ApiModelProperty(value = "备注")
	private String comments;

	@ApiModelProperty(value = "结论(收货结论0合格1不合格2待处理3拒收)")
	private String result;

	@ApiModelProperty(value = "原因")
	private String reason;

	@ApiModelProperty(value = "生产商批号")
	private String manufacturerLotNum;

	@ApiModelProperty(value = "执行者")
	private String executor;

	@ApiModelProperty(value = "容器编码")
	private String containerCode;

	@ApiModelProperty(value = "货格编码")
	private String goodsCaseCode;

	@ApiModelProperty(value = "状态")
	private String status;

	@ApiModelProperty(value = "第几次质检")
	private Integer times;

	@ApiModelProperty(value = "收货行拓展属性", hidden = true)
	private RcvLinesProfilesDTO rcvLinesProfilesDTO;

	@ApiModelProperty(value = "收货行数据", hidden = true)
	private RcvLinesDTO rcvLinesDTO;

	@ApiModelProperty(value = "数据版本号", hidden = true)
	private Integer dataVersion;

	@ApiModelProperty(value = "单品零散数(列表展示用)", position = 1)
	private BigDecimal oddPieceQty;

	@ApiModelProperty(value = "包装件数(列表展示用)", position = 1)
	private BigDecimal caseQty;

	@ApiModelProperty(value = "包装件数(优先)(列表展示用)", position = 1)
	private BigDecimal oddCaseQty;

	@ApiModelProperty(value = "商品条码(列表展示用)", position = 1)
	private String itemBarcode;
	@ApiModelProperty(value = "商品体积(列表展示用)", position = 1)
	private BigDecimal itemVolume;
	@ApiModelProperty(value = "商品重量(列表展示用)", position = 1)
	private BigDecimal itemWeight;
	@ApiModelProperty(value = "质检是否双票(列表展示用)", position = 1)
	private Integer itemHealthTwovoflag;
	@ApiModelProperty(value = "仓库名称(列表展示用)", position = 1)
	private String warehouseName;
	@ApiModelProperty(value = "商品编码(列表展示用)", position = 1)
	private String itemNum;
	@ApiModelProperty(value = "商品名称(列表展示用)", position = 1)
	private String itemName;
	@ApiModelProperty(value = "商品大类名(列表展示用)", position = 1)
	private String categoryName;

	@ApiModelProperty(value = "质检结论1(列表展示用)", position = 1)
	private String qcResult2;
	@ApiModelProperty(value = "质检原因1(列表展示用)", position = 1)
	private String qcReason2;
	@ApiModelProperty(value = "质检人员1(列表展示用)", position = 1)
	private String qcOpcode2;
	@ApiModelProperty(value = "质检1时间(列表展示用)", position = 1)
	private Date qcDate2;

	/**
	 * 企业id
	 */
	@ApiModelProperty(value = "企业id", hidden = true)
	private String enterpriseId;
	/**
	 * 创建时间
	 */
	@ApiModelProperty(value = "创建时间", hidden = true)
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date gmtCreate;

	/**
	 * 创建人编码
	 */
	@ApiModelProperty(value = "创建人编码", hidden = true)
	private String creator;

	/**
	 * 创建人用户名
	 */
	@ApiModelProperty(value = "创建人用户名", hidden = true)
	private String creatorName;

	/**
	 * 修改时间
	 */
	@ApiModelProperty(value = "修改时间", hidden = true)
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date gmtModified;

	/**
	 * 修改人编码
	 */
	@ApiModelProperty(value = "修改人编码", hidden = true)
	private String modifier;

	/**
	 * 修改人用户名
	 */
	@ApiModelProperty(value = "修改人用户名", hidden = true)
	private String modifierName;

	@ApiModelProperty(value = "数据是否锁定(0否1是拆分/异常处理并发)", hidden = true)
	private Integer isLock;

	@ApiModelProperty(value = "拆分人")
	private String spilterName;

	@ApiModelProperty(value = "是否拆分(0否1是)")
	private Integer isSpilt;

	@ApiModelProperty(value = "批次生产日期(列表展示用)", position = 9)
	private String batchProductionDate;

	@ApiModelProperty(value = "批次有效期(列表展示用)", position = 9)
	private String batchExpiredDate;

	@ApiModelProperty(value = "商品分类code名(展示用)", position = 9)
	private String categoryId;

	@ApiModelProperty(value = "仓库编码", position = 9)
	private String warehouseCode;

	// -------------------库存服务所需字段--------------------------------
	// 供应商id
	@ApiModelProperty(value = "供应商id(隐藏字段)", position = 9)
	private String vendorId;
	// 组织id
	@ApiModelProperty(value = "组织id(隐藏字段)", position = 9)
	private String orgId;
	// erp
	@ApiModelProperty(value = "erp批号(隐藏字段)", position = 9)
	private String erpBatchNum;
	// 产地
	@ApiModelProperty(value = "商品产地", position = 9)
	private String productionPlace;
	// 单品含税额
	@ApiModelProperty(value = "单品含税进价(隐藏字段)", position = 9)
	private BigDecimal piecePriceTi;
	// 单品不含税额
	@ApiModelProperty(value = "单品不含税进价(隐藏字段)", position = 9)
	private BigDecimal piecePriceTe;
	// 税率
	@ApiModelProperty(value = "税率(隐藏字段)", position = 9)
	private BigDecimal taxRate;

//	@ApiModelProperty(value = "库别编码", position = 9)
//	private String whAreaCategoryCode;

	// =================== 5.12 新增=========================================
	@ApiModelProperty(value = "区域编号(推荐库区编号)(逗号分隔列表)", position = -1)
	private String warehouseAreaCodes;
	@ApiModelProperty(value = "商品整散分离标识(0整散合一/1整散分开)", position = -1)
	private String itemIsPieceOrCase;
	@ApiModelProperty(value = "推荐库别编号(逗号分隔列表)", position = -1)
	private String warehouseAreaCategorys;

	// ===============5.17 新加字段================
	@ApiModelProperty(value = "收货类型", position = -1)
	private String rcvType;
	@ApiModelProperty(value = "商品规格说明(冗余)", position = -1)
	private String specification;
	@ApiModelProperty(value = "上市许可持有人(冗余)", position = -1)
	private String itemHealthListedLicensor;
	@ApiModelProperty(value = "生产企业许可证号(冗余)", position = -1)
	private String itemHealthProduceLicense;

	@ApiModelProperty(value = "商品是否进口(冗余)", position = -1)
	private Integer itemIsImported;
	@ApiModelProperty(value = "商品属性是否支持复检(0否1是)(冗余)", position = -1)
	private Integer itemIsRecheck;
	@ApiModelProperty(value = "商品是否器械(0否1是)(冗余)", position = -1)
	private Integer isInstrument;

	// ========5.20新加==========================
	// 商品通用名
	@ApiModelProperty(value = "商品通用名(冗余)", position = -2)
	private String itemHealthCommonName;
	// 商品是否锁定
	@ApiModelProperty(value = "商品是否锁定(0否1是)(冗余)", position = -2)
	private Integer itemIsLock;
	// 商品是否代管
	@ApiModelProperty(value = "商品是否代管(0否1是)(冗余)", position = -2)
	private Integer isHosted;
	// 生产厂家id
	@ApiModelProperty(value = "生产厂家id(冗余)", position = -2)
	private String manufacturerId;
	// 生产厂家code
	@ApiModelProperty(value = "生产厂家编码(冗余)", position = -2)
	private String manufacturerCode;
	// 生产厂家name
	@ApiModelProperty(value = "生产厂家名称(冗余)", position = -2)
	private String manufacturerName;
	@ApiModelProperty(value = "入库预留保质天数(冗余)", position = -2)
	private BigDecimal inStoreDay;
	// 有效期天数(需转换)
	@ApiModelProperty(value = "有效期天数(需转换)(冗余)", position = -2)
	private BigDecimal shelfLifeDays;
	// 保质期单位
	@ApiModelProperty(value = "保质期单位(年/月/日,默认日)(冗余)", position = -2)
	private String shelfLifeUnit;
	// 是否特殊品
	@ApiModelProperty(value = "商品是否特殊品(0否1是)(冗余)", position = -2)
	private Integer itemIsSpecial;
	// 是否无菌
	@ApiModelProperty(value = "商品是否无菌(0否1是)(冗余)", position = -2)
	private Integer isBacteria;
	// 退货单位提供的商品售出期间储存、运输质量控制情况
	@ApiModelProperty(value = "退货单位提供的商品售出期间储存、运输质量控制情况(冗余)", position = -2)
	private String returnInfo;
	// 退货冷藏品出库期质量
	@ApiModelProperty(value = "退货冷藏品出库期质量(冗余)", position = -2)
	private String returnColdQuality;
	// 退货说明
	@ApiModelProperty(value = "退货说明(冗余)", position = -2)
	private String returnExplain;

	@ApiModelProperty(value = "是否已复检(0否1是)", position = -2)
	private Integer isRechecked;

	@ApiModelProperty(value = "是否抽样过(0否1是)", position = -2)
	private Integer isSpotCheck;

	@ApiModelProperty(value = "是否监管码扫描(0否1是默认0)", position = -2)
	private Integer isSupervise;

	@ApiModelProperty(value = "选择的库区名称", position = -3)
	private String warehouseAreaName;
	@ApiModelProperty(value = "选择的库区id", position = -3)
	private String warehouseAreaId;
	@ApiModelProperty(value = "选择的库区编号", position = -3)
	private String warehouseAreaCode;
	@ApiModelProperty(value = "选择的建筑物id", position = -3)
	private String bldgId;
	@ApiModelProperty(value = "选择的建筑物名称", position = -3)
	private String bldgName;
	@ApiModelProperty(value = "选择的建筑物编号", position = -3)
	private String bldgCode;

	// ==== 6.1 新加字段===========================================================
	@ApiModelProperty(value = "受托生产企业id(供应商id)", position = -4)
	private String delegationPartyId;
	@ApiModelProperty(value = "受托生产企业名称(供应商名称)", position = -4)
	private String delegationPartyName;
	@ApiModelProperty(value = "是否委托生产(0否1是)", position = -4)
	private Integer isDelegate;

	// ==== 6.9 新加字段===========================================================
	@ApiModelProperty(value = "显示生产日期", position = -4)
	private String showProductionDate;
	@ApiModelProperty(value = "显示有效期至", position = -4)
	private String showExpiredDate;
	@ApiModelProperty(value = "最小包装数", position = -4)
	private BigDecimal pieceConversionRate;
	@ApiModelProperty(value = "推荐库房列表", position = -4)
	private String warehouseRoomCodes;

	// ============6.10新加字段======================================================
	@ApiModelProperty(value = "asn单行明细备注", position = -4)
	private String asnLineRemark;

	// ============6.11新加字段======================================================
	@ApiModelProperty(value = "商品助记码", position = -4)
	private String itemZjm;

	// 来源行id
	@ApiModelProperty(value = "入库来源行id", position = -5)
	private String refLineId;

	// 来源类型
	@ApiModelProperty(value = "入库来源类型(0收货1质检2复检)", position = -5)
	private Integer refType;

	// 处理意见
	@ApiModelProperty(value = "处理意见(来源于复检)", position = -5)
	private String handlingSuggestion;

	@ApiModelProperty(value = "存储条件", position = -6)
	private String storeTemperature;

	// ========6.3新加 tianahailong==========================

	@ApiModelProperty(value = "标准托盘量")
	private String palletStdQuantity;

	@ApiModelProperty(value = "应扫")
	private Integer shouldScanQty;

	@ApiModelProperty(value = "复检结果")
	private String reCheckResult;
	@ApiModelProperty(value = "双人业务类型(0:正常 1：双人验收，默认正常)")
	private Integer itemHealthIsDouble;
	@ApiModelProperty(value = "是否追溯(0否1是)默认0", position = -5)
	private Integer isRetrospect;

	// 明细表信息：是否追溯，是否双人验收，商品编码，商品名称，商品规格，商品大类，生产厂家，产地，上市许可证持有人，包装单位，包装含量（改为件包装），容器号、批号、显示生产日期（当前为生产日期）、显示有效期至，收货数量，收货件数，收货零散数，收货结论
//	，明细的处理原因、不足原因、不足原因补充，存储分类，批准文号，注册证号
	@ApiModelProperty(value = "(收货)处理原因", position = -100)
	private String rcvReason1;

	@ApiModelProperty(value = "(收货)不足原因", position = -100)
	private String rcvReason2;

	@ApiModelProperty(value = "(收货)不足原因补充", position = -100)
	private String rcvReason3;

	@ApiModelProperty(value = "(收货)退货原因", position = -100)
	private String rcvRtReason;
	@ApiModelProperty(value = "存储分类(整散分离属性)", position = -100)
	private String itemHealthStoreCategory;
	// 注册证号
	@ApiModelProperty(value = "注册证号(冗余)", position = -100)
	private String itemHealthRegisterNbr;
	@ApiModelProperty(value = "商品批准文号", position = -1113)
	private String approvalNumber;
//	生产企业注册证号
	@ApiModelProperty(value = "生产企业注册证号", position = -1113)
	private String manufacturerRegisterNo;

	@ApiModelProperty(value = "处理原因(质检初始,码表)", position = -200)
	private String handlingReason;
	@ApiModelProperty(value = "不足原因补充(质检初始)", position = -200)
	private String defReasonExt;
	@ApiModelProperty(value = "不足原因(质检初始)", position = -200)
	private String defReason;
	@ApiModelProperty(value = "处理措施(质检初始)", position = -200)
	private String handlingMeasure;
	@ApiModelProperty(value = "结论(0合格1不合格2待处理3拒收)", position = -200)
	private String stockStatus;
	@ApiModelProperty(value = "任务数据来源(0收货/1一次质检/2复检)", position = -200)
	private String sourceType;
	@ApiModelProperty(value = "上一节点行id(用于记录上一次复检的行id)", position = -200)
	private String preNodeLineId;

	@ApiModelProperty(value = "质检报告单号(关联批次)", position = -201)
	private String rqcReportNo;

	@ApiModelProperty(value = "小数拆分标识(Y为允许,其他均不可)", position = -2)
	private String allowDecimal;

	@ApiModelProperty(value = "商品条件查询", position = -2)
	private String itemCondition;

	// 8.27新加
	@ApiModelProperty(value = "(新)货位id/用于调账选择", position = -2017)
	private String whLocatorId;
	@ApiModelProperty(value = "(新)货位code/用于调账选择", position = -2017)
	private String whLocatorCode;
	@ApiModelProperty(value = "(新)库区code/用于调账选择", position = -2017)
	private String whAreaCode;
	@ApiModelProperty(value = "(新)库别code/用于调账选择", position = -2017)
	private String whAreaCategoryCode;
	@ApiModelProperty(value = "(新)代管货位id", position = -2017)
	private String hostLocatorId;

	@ApiModelProperty(value = "单据类型")
	private String billType;

	@ApiModelProperty(value = "asn单号(ASN)", hidden = true)
	private String asnAutoBillNum;

	@ApiModelProperty(value = "供应商名称(查询字段)", position = 1)
	private String vendorName;

	@ApiModelProperty(value = "长", position = 1)
	private BigDecimal length;

	@ApiModelProperty(value = "宽", position = 1)
	private BigDecimal width;

	@ApiModelProperty(value = "高", position = 1)
	private BigDecimal height;

	public String getAsnAutoBillNum() {
		return asnAutoBillNum;
	}

	public void setAsnAutoBillNum(String asnAutoBillNum) {
		this.asnAutoBillNum = asnAutoBillNum;
	}

	public String getVendorName() {
		return vendorName;
	}

	public void setVendorName(String vendorName) {
		this.vendorName = vendorName;
	}

	public String getWhLocatorId() {
		return whLocatorId;
	}

	public String getHostLocatorId() {
		return hostLocatorId;
	}

	public void setHostLocatorId(String hostLocatorId) {
		this.hostLocatorId = hostLocatorId;
	}

	public void setWhLocatorId(String whLocatorId) {
		this.whLocatorId = whLocatorId;
	}

	public String getWhLocatorCode() {
		return whLocatorCode;
	}

	public void setWhLocatorCode(String whLocatorCode) {
		this.whLocatorCode = whLocatorCode;
	}

	public String getWhAreaCode() {
		return whAreaCode;
	}

	public void setWhAreaCode(String whAreaCode) {
		this.whAreaCode = whAreaCode;
	}

	public String getHandlingMeasure() {
		return handlingMeasure;
	}

	public String getItemCondition() {
		return itemCondition;
	}

	public void setItemCondition(String itemCondition) {
		this.itemCondition = itemCondition;
	}

	public void setHandlingMeasure(String handlingMeasure) {
		this.handlingMeasure = handlingMeasure;
	}

	public String getAllowDecimal() {
		return allowDecimal;
	}

	public void setAllowDecimal(String allowDecimal) {
		this.allowDecimal = allowDecimal;
	}

	public String getRqcReportNo() {
		return rqcReportNo;
	}

	public void setRqcReportNo(String rqcReportNo) {
		this.rqcReportNo = rqcReportNo;
	}

	public String getPreNodeLineId() {
		return preNodeLineId;
	}

	public void setPreNodeLineId(String preNodeLineId) {
		this.preNodeLineId = preNodeLineId;
	}

	public String getStockStatus() {
		return stockStatus;
	}

	public void setStockStatus(String stockStatus) {
		this.stockStatus = stockStatus;
	}

	public String getSourceType() {
		return sourceType;
	}

	public void setSourceType(String sourceType) {
		this.sourceType = sourceType;
	}

	public String getHandlingReason() {
		return handlingReason;
	}

	public void setHandlingReason(String handlingReason) {
		this.handlingReason = handlingReason;
	}

	public String getDefReasonExt() {
		return defReasonExt;
	}

	public void setDefReasonExt(String defReasonExt) {
		this.defReasonExt = defReasonExt;
	}

	public String getDefReason() {
		return defReason;
	}

	public void setDefReason(String defReason) {
		this.defReason = defReason;
	}

	public String getManufacturerRegisterNo() {
		return manufacturerRegisterNo;
	}

	public void setManufacturerRegisterNo(String manufacturerRegisterNo) {
		this.manufacturerRegisterNo = manufacturerRegisterNo;
	}

	public String getTaskRqcId() {
		return taskRqcId;
	}

	public void setTaskRqcId(String taskRqcId) {
		this.taskRqcId = taskRqcId;
	}

	public String getMtlOwnerId() {
		return mtlOwnerId;
	}

	public void setMtlOwnerId(String mtlOwnerId) {
		this.mtlOwnerId = mtlOwnerId;
	}

	public String getWarehouseId() {
		return warehouseId;
	}

	public void setWarehouseId(String warehouseId) {
		this.warehouseId = warehouseId;
	}

	public String getSrcBillType() {
		return srcBillType;
	}

	public void setSrcBillType(String srcBillType) {
		this.srcBillType = srcBillType;
	}

	public String getSrcHeaderId() {
		return srcHeaderId;
	}

	public void setSrcHeaderId(String srcHeaderId) {
		this.srcHeaderId = srcHeaderId;
	}

	public String getSrcLineId() {
		return srcLineId;
	}

	public void setSrcLineId(String srcLineId) {
		this.srcLineId = srcLineId;
	}

	public String getAsnHeaderId() {
		return asnHeaderId;
	}

	public void setAsnHeaderId(String asnHeaderId) {
		this.asnHeaderId = asnHeaderId;
	}

	public String getAsnLineId() {
		return asnLineId;
	}

	public void setAsnLineId(String asnLineId) {
		this.asnLineId = asnLineId;
	}

	public String getRcvHeaderId() {
		return rcvHeaderId;
	}

	public void setRcvHeaderId(String rcvHeaderId) {
		this.rcvHeaderId = rcvHeaderId;
	}

	public String getRcvLineId() {
		return rcvLineId;
	}

	public void setRcvLineId(String rcvLineId) {
		this.rcvLineId = rcvLineId;
	}

	public String getItemId() {
		return itemId;
	}

	public void setItemId(String itemId) {
		this.itemId = itemId;
	}

	public BigDecimal getQty() {
		return qty;
	}

	public void setQty(BigDecimal qty) {
		this.qty = qty;
	}

	public String getUom() {
		return uom;
	}

	public void setUom(String uom) {
		this.uom = uom;
	}

	public BigDecimal getConversionRate() {
		return conversionRate;
	}

	public void setConversionRate(BigDecimal conversionRate) {
		this.conversionRate = conversionRate;
	}

	public BigDecimal getPieceQty() {
		return pieceQty;
	}

	public void setPieceQty(BigDecimal pieceQty) {
		this.pieceQty = pieceQty;
	}

	public String getBatchId() {
		return batchId;
	}

	public void setBatchId(String batchId) {
		this.batchId = batchId;
	}

	public String getBatchNum() {
		return batchNum;
	}

	public void setBatchNum(String batchNum) {
		this.batchNum = batchNum;
	}

	public String getVendorBatchNum() {
		return vendorBatchNum;
	}

	public void setVendorBatchNum(String vendorBatchNum) {
		this.vendorBatchNum = vendorBatchNum;
	}

	public String getActiveFlag() {
		return activeFlag;
	}

	public void setActiveFlag(String activeFlag) {
		this.activeFlag = activeFlag;
	}

	public BigDecimal getRemainQty() {
		return remainQty;
	}

	public void setRemainQty(BigDecimal remainQty) {
		this.remainQty = remainQty;
	}

	public String getComments() {
		return comments;
	}

	public void setComments(String comments) {
		this.comments = comments;
	}

	public String getResult() {
		return result;
	}

	public void setResult(String result) {
		this.result = result;
	}

	public String getReason() {
		return reason;
	}

	public void setReason(String reason) {
		this.reason = reason;
	}

	public String getManufacturerLotNum() {
		return manufacturerLotNum;
	}

	public void setManufacturerLotNum(String manufacturerLotNum) {
		this.manufacturerLotNum = manufacturerLotNum;
	}

	public String getExecutor() {
		return executor;
	}

	public void setExecutor(String executor) {
		this.executor = executor;
	}

	public String getContainerCode() {
		return containerCode;
	}

	public void setContainerCode(String containerCode) {
		this.containerCode = containerCode;
	}

	public String getGoodsCaseCode() {
		return goodsCaseCode;
	}

	public void setGoodsCaseCode(String goodsCaseCode) {
		this.goodsCaseCode = goodsCaseCode;
	}

	public String getStatus() {
		return status;
	}

	public void setStatus(String status) {
		this.status = status;
	}

	public Integer getTimes() {
		return times;
	}

	public void setTimes(Integer times) {
		this.times = times;
	}

	public RcvLinesProfilesDTO getRcvLinesProfilesDTO() {
		return rcvLinesProfilesDTO;
	}

	public void setRcvLinesProfilesDTO(RcvLinesProfilesDTO rcvLinesProfilesDTO) {
		this.rcvLinesProfilesDTO = rcvLinesProfilesDTO;
	}

	public RcvLinesDTO getRcvLinesDTO() {
		return rcvLinesDTO;
	}

	public void setRcvLinesDTO(RcvLinesDTO rcvLinesDTO) {
		this.rcvLinesDTO = rcvLinesDTO;
	}

	public Integer getDataVersion() {
		return dataVersion;
	}

	public void setDataVersion(Integer dataVersion) {
		this.dataVersion = dataVersion;
	}

	public BigDecimal getOddPieceQty() {
		return oddPieceQty;
	}

	public void setOddPieceQty(BigDecimal oddPieceQty) {
		this.oddPieceQty = oddPieceQty;
	}

	public BigDecimal getCaseQty() {
		return caseQty;
	}

	public void setCaseQty(BigDecimal caseQty) {
		this.caseQty = caseQty;
	}

	public BigDecimal getOddCaseQty() {
		return oddCaseQty;
	}

	public void setOddCaseQty(BigDecimal oddCaseQty) {
		this.oddCaseQty = oddCaseQty;
	}

	public String getItemBarcode() {
		return itemBarcode;
	}

	public void setItemBarcode(String itemBarcode) {
		this.itemBarcode = itemBarcode;
	}

	public BigDecimal getItemVolume() {
		return itemVolume;
	}

	public void setItemVolume(BigDecimal itemVolume) {
		this.itemVolume = itemVolume;
	}

	public BigDecimal getItemWeight() {
		return itemWeight;
	}

	public void setItemWeight(BigDecimal itemWeight) {
		this.itemWeight = itemWeight;
	}

	public Integer getItemHealthTwovoflag() {
		return itemHealthTwovoflag;
	}

	public void setItemHealthTwovoflag(Integer itemHealthTwovoflag) {
		this.itemHealthTwovoflag = itemHealthTwovoflag;
	}

	public String getWarehouseName() {
		return warehouseName;
	}

	public void setWarehouseName(String warehouseName) {
		this.warehouseName = warehouseName;
	}

	public String getItemNum() {
		return itemNum;
	}

	public void setItemNum(String itemNum) {
		this.itemNum = itemNum;
	}

	public String getItemName() {
		return itemName;
	}

	public void setItemName(String itemName) {
		this.itemName = itemName;
	}

	public String getCategoryName() {
		return categoryName;
	}

	public void setCategoryName(String categoryName) {
		this.categoryName = categoryName;
	}

	public String getQcResult2() {
		return qcResult2;
	}

	public void setQcResult2(String qcResult2) {
		this.qcResult2 = qcResult2;
	}

	public String getQcReason2() {
		return qcReason2;
	}

	public void setQcReason2(String qcReason2) {
		this.qcReason2 = qcReason2;
	}

	public String getQcOpcode2() {
		return qcOpcode2;
	}

	public void setQcOpcode2(String qcOpcode2) {
		this.qcOpcode2 = qcOpcode2;
	}

	public Date getQcDate2() {
		return qcDate2;
	}

	public void setQcDate2(Date qcDate2) {
		this.qcDate2 = qcDate2;
	}

	public String getEnterpriseId() {
		return enterpriseId;
	}

	public void setEnterpriseId(String enterpriseId) {
		this.enterpriseId = enterpriseId;
	}

	public Date getGmtCreate() {
		return gmtCreate;
	}

	public void setGmtCreate(Date gmtCreate) {
		this.gmtCreate = gmtCreate;
	}

	public String getCreator() {
		return creator;
	}

	public void setCreator(String creator) {
		this.creator = creator;
	}

	public String getCreatorName() {
		return creatorName;
	}

	public void setCreatorName(String creatorName) {
		this.creatorName = creatorName;
	}

	public Date getGmtModified() {
		return gmtModified;
	}

	public void setGmtModified(Date gmtModified) {
		this.gmtModified = gmtModified;
	}

	public String getModifier() {
		return modifier;
	}

	public void setModifier(String modifier) {
		this.modifier = modifier;
	}

	public String getModifierName() {
		return modifierName;
	}

	public void setModifierName(String modifierName) {
		this.modifierName = modifierName;
	}

	public Integer getIsLock() {
		return isLock;
	}

	public void setIsLock(Integer isLock) {
		this.isLock = isLock;
	}

	public String getSpilterName() {
		return spilterName;
	}

	public void setSpilterName(String spilterName) {
		this.spilterName = spilterName;
	}

	public Integer getIsSpilt() {
		return isSpilt;
	}

	public void setIsSpilt(Integer isSpilt) {
		this.isSpilt = isSpilt;
	}

	public String getBatchProductionDate() {
		return batchProductionDate;
	}

	public void setBatchProductionDate(String batchProductionDate) {
		this.batchProductionDate = batchProductionDate;
	}

	public String getBatchExpiredDate() {
		return batchExpiredDate;
	}

	public void setBatchExpiredDate(String batchExpiredDate) {
		this.batchExpiredDate = batchExpiredDate;
	}

	public String getCategoryId() {
		return categoryId;
	}

	public void setCategoryId(String categoryId) {
		this.categoryId = categoryId;
	}

	public String getWarehouseCode() {
		return warehouseCode;
	}

	public void setWarehouseCode(String warehouseCode) {
		this.warehouseCode = warehouseCode;
	}

	public String getVendorId() {
		return vendorId;
	}

	public void setVendorId(String vendorId) {
		this.vendorId = vendorId;
	}

	public String getOrgId() {
		return orgId;
	}

	public void setOrgId(String orgId) {
		this.orgId = orgId;
	}

	public String getErpBatchNum() {
		return erpBatchNum;
	}

	public void setErpBatchNum(String erpBatchNum) {
		this.erpBatchNum = erpBatchNum;
	}

	public String getProductionPlace() {
		return productionPlace;
	}

	public void setProductionPlace(String productionPlace) {
		this.productionPlace = productionPlace;
	}

	public BigDecimal getPiecePriceTi() {
		return piecePriceTi;
	}

	public void setPiecePriceTi(BigDecimal piecePriceTi) {
		this.piecePriceTi = piecePriceTi;
	}

	public BigDecimal getPiecePriceTe() {
		return piecePriceTe;
	}

	public void setPiecePriceTe(BigDecimal piecePriceTe) {
		this.piecePriceTe = piecePriceTe;
	}

	public BigDecimal getTaxRate() {
		return taxRate;
	}

	public void setTaxRate(BigDecimal taxRate) {
		this.taxRate = taxRate;
	}

	public String getWhAreaCategoryCode() {
		return whAreaCategoryCode;
	}

	public void setWhAreaCategoryCode(String whAreaCategoryCode) {
		this.whAreaCategoryCode = whAreaCategoryCode;
	}

	public String getWarehouseAreaCodes() {
		return warehouseAreaCodes;
	}

	public void setWarehouseAreaCodes(String warehouseAreaCodes) {
		this.warehouseAreaCodes = warehouseAreaCodes;
	}

	public String getItemIsPieceOrCase() {
		return itemIsPieceOrCase;
	}

	public void setItemIsPieceOrCase(String itemIsPieceOrCase) {
		this.itemIsPieceOrCase = itemIsPieceOrCase;
	}

	public String getWarehouseAreaCategorys() {
		return warehouseAreaCategorys;
	}

	public void setWarehouseAreaCategorys(String warehouseAreaCategorys) {
		this.warehouseAreaCategorys = warehouseAreaCategorys;
	}

	public String getRcvType() {
		return rcvType;
	}

	public void setRcvType(String rcvType) {
		this.rcvType = rcvType;
	}

	public String getSpecification() {
		return specification;
	}

	public void setSpecification(String specification) {
		this.specification = specification;
	}

	public String getItemHealthListedLicensor() {
		return itemHealthListedLicensor;
	}

	public void setItemHealthListedLicensor(String itemHealthListedLicensor) {
		this.itemHealthListedLicensor = itemHealthListedLicensor;
	}

	public String getItemHealthProduceLicense() {
		return itemHealthProduceLicense;
	}

	public void setItemHealthProduceLicense(String itemHealthProduceLicense) {
		this.itemHealthProduceLicense = itemHealthProduceLicense;
	}

	public Integer getItemIsImported() {
		return itemIsImported;
	}

	public void setItemIsImported(Integer itemIsImported) {
		this.itemIsImported = itemIsImported;
	}

	public Integer getItemIsRecheck() {
		return itemIsRecheck;
	}

	public void setItemIsRecheck(Integer itemIsRecheck) {
		this.itemIsRecheck = itemIsRecheck;
	}

	public Integer getIsInstrument() {
		return isInstrument;
	}

	public void setIsInstrument(Integer isInstrument) {
		this.isInstrument = isInstrument;
	}

	public String getItemHealthCommonName() {
		return itemHealthCommonName;
	}

	public void setItemHealthCommonName(String itemHealthCommonName) {
		this.itemHealthCommonName = itemHealthCommonName;
	}

	public Integer getItemIsLock() {
		return itemIsLock;
	}

	public void setItemIsLock(Integer itemIsLock) {
		this.itemIsLock = itemIsLock;
	}

	public Integer getIsHosted() {
		return isHosted;
	}

	public void setIsHosted(Integer isHosted) {
		this.isHosted = isHosted;
	}

	public String getManufacturerId() {
		return manufacturerId;
	}

	public void setManufacturerId(String manufacturerId) {
		this.manufacturerId = manufacturerId;
	}

	public String getManufacturerCode() {
		return manufacturerCode;
	}

	public void setManufacturerCode(String manufacturerCode) {
		this.manufacturerCode = manufacturerCode;
	}

	public String getManufacturerName() {
		return manufacturerName;
	}

	public void setManufacturerName(String manufacturerName) {
		this.manufacturerName = manufacturerName;
	}

	public BigDecimal getInStoreDay() {
		return inStoreDay;
	}

	public void setInStoreDay(BigDecimal inStoreDay) {
		this.inStoreDay = inStoreDay;
	}

	public BigDecimal getShelfLifeDays() {
		return shelfLifeDays;
	}

	public void setShelfLifeDays(BigDecimal shelfLifeDays) {
		this.shelfLifeDays = shelfLifeDays;
	}

	public String getShelfLifeUnit() {
		return shelfLifeUnit;
	}

	public void setShelfLifeUnit(String shelfLifeUnit) {
		this.shelfLifeUnit = shelfLifeUnit;
	}

	public Integer getItemIsSpecial() {
		return itemIsSpecial;
	}

	public void setItemIsSpecial(Integer itemIsSpecial) {
		this.itemIsSpecial = itemIsSpecial;
	}

	public Integer getIsBacteria() {
		return isBacteria;
	}

	public void setIsBacteria(Integer isBacteria) {
		this.isBacteria = isBacteria;
	}

	public String getReturnInfo() {
		return returnInfo;
	}

	public void setReturnInfo(String returnInfo) {
		this.returnInfo = returnInfo;
	}

	public String getReturnColdQuality() {
		return returnColdQuality;
	}

	public void setReturnColdQuality(String returnColdQuality) {
		this.returnColdQuality = returnColdQuality;
	}

	public String getReturnExplain() {
		return returnExplain;
	}

	public void setReturnExplain(String returnExplain) {
		this.returnExplain = returnExplain;
	}

	public Integer getIsRechecked() {
		return isRechecked;
	}

	public void setIsRechecked(Integer isRechecked) {
		this.isRechecked = isRechecked;
	}

	public Integer getIsSpotCheck() {
		return isSpotCheck;
	}

	public void setIsSpotCheck(Integer isSpotCheck) {
		this.isSpotCheck = isSpotCheck;
	}

	public Integer getIsSupervise() {
		return isSupervise;
	}

	public void setIsSupervise(Integer isSupervise) {
		this.isSupervise = isSupervise;
	}

	public String getWarehouseAreaName() {
		return warehouseAreaName;
	}

	public void setWarehouseAreaName(String warehouseAreaName) {
		this.warehouseAreaName = warehouseAreaName;
	}

	public String getWarehouseAreaId() {
		return warehouseAreaId;
	}

	public void setWarehouseAreaId(String warehouseAreaId) {
		this.warehouseAreaId = warehouseAreaId;
	}

	public String getWarehouseAreaCode() {
		return warehouseAreaCode;
	}

	public void setWarehouseAreaCode(String warehouseAreaCode) {
		this.warehouseAreaCode = warehouseAreaCode;
	}

	public String getBldgId() {
		return bldgId;
	}

	public void setBldgId(String bldgId) {
		this.bldgId = bldgId;
	}

	public String getBldgName() {
		return bldgName;
	}

	public void setBldgName(String bldgName) {
		this.bldgName = bldgName;
	}

	public String getBldgCode() {
		return bldgCode;
	}

	public void setBldgCode(String bldgCode) {
		this.bldgCode = bldgCode;
	}

	public String getDelegationPartyId() {
		return delegationPartyId;
	}

	public void setDelegationPartyId(String delegationPartyId) {
		this.delegationPartyId = delegationPartyId;
	}

	public String getDelegationPartyName() {
		return delegationPartyName;
	}

	public void setDelegationPartyName(String delegationPartyName) {
		this.delegationPartyName = delegationPartyName;
	}

	public Integer getIsDelegate() {
		return isDelegate;
	}

	public void setIsDelegate(Integer isDelegate) {
		this.isDelegate = isDelegate;
	}

	public String getShowProductionDate() {
		return showProductionDate;
	}

	public void setShowProductionDate(String showProductionDate) {
		this.showProductionDate = showProductionDate;
	}

	public String getShowExpiredDate() {
		return showExpiredDate;
	}

	public void setShowExpiredDate(String showExpiredDate) {
		this.showExpiredDate = showExpiredDate;
	}

	public BigDecimal getPieceConversionRate() {
		return pieceConversionRate;
	}

	public void setPieceConversionRate(BigDecimal pieceConversionRate) {
		this.pieceConversionRate = pieceConversionRate;
	}

	public String getWarehouseRoomCodes() {
		return warehouseRoomCodes;
	}

	public void setWarehouseRoomCodes(String warehouseRoomCodes) {
		this.warehouseRoomCodes = warehouseRoomCodes;
	}

	public String getAsnLineRemark() {
		return asnLineRemark;
	}

	public void setAsnLineRemark(String asnLineRemark) {
		this.asnLineRemark = asnLineRemark;
	}

	public String getItemZjm() {
		return itemZjm;
	}

	public void setItemZjm(String itemZjm) {
		this.itemZjm = itemZjm;
	}

	public String getRefLineId() {
		return refLineId;
	}

	public void setRefLineId(String refLineId) {
		this.refLineId = refLineId;
	}

	public Integer getRefType() {
		return refType;
	}

	public void setRefType(Integer refType) {
		this.refType = refType;
	}

	public String getHandlingSuggestion() {
		return handlingSuggestion;
	}

	public void setHandlingSuggestion(String handlingSuggestion) {
		this.handlingSuggestion = handlingSuggestion;
	}

	public String getStoreTemperature() {
		return storeTemperature;
	}

	public void setStoreTemperature(String storeTemperature) {
		this.storeTemperature = storeTemperature;
	}

	public String getPalletStdQuantity() {
		return palletStdQuantity;
	}

	public void setPalletStdQuantity(String palletStdQuantity) {
		this.palletStdQuantity = palletStdQuantity;
	}

	public Integer getShouldScanQty() {
		return shouldScanQty;
	}

	public void setShouldScanQty(Integer shouldScanQty) {
		this.shouldScanQty = shouldScanQty;
	}

	public String getReCheckResult() {
		return reCheckResult;
	}

	public void setReCheckResult(String reCheckResult) {
		this.reCheckResult = reCheckResult;
	}

	public Integer getItemHealthIsDouble() {
		return itemHealthIsDouble;
	}

	public void setItemHealthIsDouble(Integer itemHealthIsDouble) {
		this.itemHealthIsDouble = itemHealthIsDouble;
	}

	public Integer getIsRetrospect() {
		return isRetrospect;
	}

	public void setIsRetrospect(Integer isRetrospect) {
		this.isRetrospect = isRetrospect;
	}

	public String getRcvReason1() {
		return rcvReason1;
	}

	public void setRcvReason1(String rcvReason1) {
		this.rcvReason1 = rcvReason1;
	}

	public String getRcvReason2() {
		return rcvReason2;
	}

	public void setRcvReason2(String rcvReason2) {
		this.rcvReason2 = rcvReason2;
	}

	public String getRcvReason3() {
		return rcvReason3;
	}

	public void setRcvReason3(String rcvReason3) {
		this.rcvReason3 = rcvReason3;
	}

	public String getRcvRtReason() {
		return rcvRtReason;
	}

	public void setRcvRtReason(String rcvRtReason) {
		this.rcvRtReason = rcvRtReason;
	}

	public String getItemHealthStoreCategory() {
		return itemHealthStoreCategory;
	}

	public void setItemHealthStoreCategory(String itemHealthStoreCategory) {
		this.itemHealthStoreCategory = itemHealthStoreCategory;
	}

	public String getItemHealthRegisterNbr() {
		return itemHealthRegisterNbr;
	}

	public void setItemHealthRegisterNbr(String itemHealthRegisterNbr) {
		this.itemHealthRegisterNbr = itemHealthRegisterNbr;
	}

	public String getApprovalNumber() {
		return approvalNumber;
	}

	public void setApprovalNumber(String approvalNumber) {
		this.approvalNumber = approvalNumber;
	}

	@Override
	public String getBillType() {
		return billType;
	}

	@Override
	public void setBillType(String billType) {
		this.billType = billType;
	}

	public BigDecimal getLength() {
		return length;
	}

	public void setLength(BigDecimal length) {
		this.length = length;
	}

	public BigDecimal getWidth() {
		return width;
	}

	public void setWidth(BigDecimal width) {
		this.width = width;
	}

	public BigDecimal getHeight() {
		return height;
	}

	public void setHeight(BigDecimal height) {
		this.height = height;
	}

	public BigDecimal getReceiveQty() {
		return receiveQty;
	}

	public void setReceiveQty(BigDecimal receiveQty) {
		this.receiveQty = receiveQty;
	}
}
