package com.prolog.cs.biz.inbound.dto.task;

import com.prolog.cs.biz.inbound.constant.ValidatorGroup.validAdd;
import com.prolog.cs.biz.inbound.constant.ValidatorGroup.validEdit;
import com.prolog.cs.biz.inbound.dto.rcv.RcvLinesDTO;
import com.prolog.cs.biz.inbound.dto.rcv.RcvLinesProfilesDTO;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * @Description 复检任务池
 * @Author Xiong
 * @Date 2021-04-02
 */

@ApiModel
public class TaskQcRecheckDetailRFDTO implements Serializable {
	private static final long serialVersionUID = 1398953434588887226L;
	@ApiModelProperty(value = "ID(主键雪花id)", required = true)
	@NotBlank(message = "ID(主键雪花id) can not blank!", groups = validEdit.class)
	private String taskRecheckId;

	@ApiModelProperty(value = "货主ID")
	private String mtlOwnerId;

	@ApiModelProperty(value = "仓库ID", required = true)
	@NotBlank(message = "仓库ID can not blank!", groups = validAdd.class)
	private String warehouseId;

	@ApiModelProperty(value = "来源单类型")
	private String srcBillType;

	@ApiModelProperty(value = "来源单头ID")
	private String srcHeaderId;

	@ApiModelProperty(value = "来源单行ID")
	private String srcLineId;

	@ApiModelProperty(value = "ASN头ID")
	private String asnHeaderId;

	@ApiModelProperty(value = "ASN行ID")
	private String asnLineId;

	@ApiModelProperty(value = "收货单头ID")
	private String rcvHeaderId;

	@ApiModelProperty(value = "收货行头ID")
	private String rcvLineId;

	@ApiModelProperty(value = "商品ID")
	private String itemId;

	@ApiModelProperty(value = "数量")
	private BigDecimal qty;

	@ApiModelProperty(value = "包装单位", required = true)
	@NotBlank(message = "包装单位 can not blank!", groups = validAdd.class)
	private String uom;

	@ApiModelProperty(value = "包装含量", required = true)
	@NotNull(message = "包装含量 can not blank!", groups = validAdd.class)
	private BigDecimal conversionRate;

	@ApiModelProperty(value = "单品数量", required = true)
	@NotNull(message = "单品数量 can not blank!", groups = validAdd.class)
	private BigDecimal pieceQty;

	@ApiModelProperty(value = "批次ID")
	private String batchId;

	@ApiModelProperty(value = "批次号")
	private String batchNum;

	@ApiModelProperty(value = "批号")
	private String vendorBatchNum;

	@ApiModelProperty(value = "是否允许作业标识", required = true)
	@NotBlank(message = "是否允许作业标识 can not blank!", groups = validAdd.class)
	private String activeFlag;

	@ApiModelProperty(value = "余量")
	private BigDecimal remainQty;

	@ApiModelProperty(value = "第几次复检")
	private Integer times;

	@ApiModelProperty(value = "备注")
	private String comments;

	@ApiModelProperty(value = "任务结论")
	private String result;

	/*
	 * @ApiModelProperty(value = "上一节点结论") private String stockStatus;
	 */

	@ApiModelProperty(value = "原因")
	private String reason;

	@ApiModelProperty(value = "生产商批号")
	private String manufacturerLotNum;

	@ApiModelProperty(value = "执行者")
	private String executor;

	@ApiModelProperty(value = "容器编码")
	private String containerCode;

	@ApiModelProperty(value = "货格编码")
	private String goodsCaseCode;

	@ApiModelProperty(value = "状态")
	private String status;

	@ApiModelProperty(value = "收货行拓展属性", hidden = true)
	private RcvLinesProfilesDTO rcvLinesProfilesDTO;

	@ApiModelProperty(value = "收货行数据", hidden = true)
	private RcvLinesDTO rcvLinesDTO;

	@ApiModelProperty(value = "单品零散数(列表展示用)", position = 1)
	private BigDecimal oddPieceQty;
	@ApiModelProperty(value = "包装件数(列表展示用)", position = 1)
	private BigDecimal caseQty;
	@ApiModelProperty(value = "商品条码(列表展示用)", position = 1)
	private String itemBarcode;
	@ApiModelProperty(value = "商品体积(列表展示用)", position = 1)
	private BigDecimal itemVolume;
	@ApiModelProperty(value = "商品重量(列表展示用)", position = 1)
	private BigDecimal itemWeight;
	@ApiModelProperty(value = "质检是否双票(列表展示用)", position = 1)
	private Integer itemHealthTwovoflag;
	@ApiModelProperty(value = "仓库名称(列表展示用)", position = 1)
	private String warehouseName;

	@ApiModelProperty(value = "商品编码(列表展示用)", position = 1)
	private String itemNum;
	@ApiModelProperty(value = "商品名称(列表展示用)", position = 1)
	private String itemName;
	@ApiModelProperty(value = "商品分类名(列表展示用)", position = 1)
	private String categoryName;

	/**
	 * 企业id
	 */
	@ApiModelProperty(value = "企业id", hidden = true)
	private String enterpriseId;
	/**
	 * 创建时间
	 */
	@ApiModelProperty(value = "创建时间", hidden = true)
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date gmtCreate;

	/**
	 * 创建人编码
	 */
	@ApiModelProperty(value = "创建人编码", hidden = true)
	private String creator;

	/**
	 * 创建人用户名
	 */
	@ApiModelProperty(value = "创建人用户名", hidden = true)
	private String creatorName;

	/**
	 * 修改时间
	 */
	@ApiModelProperty(value = "修改时间", hidden = true)
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date gmtModified;

	/**
	 * 修改人编码
	 */
	@ApiModelProperty(value = "修改人编码", hidden = true)
	private String modifier;

	/**
	 * 修改人用户名
	 */
	@ApiModelProperty(value = "修改人用户名", hidden = true)
	private String modifierName;

	@ApiModelProperty(value = "数据版本号", hidden = true)
	private Integer dataVersion;

	@ApiModelProperty(value = "双人业务类型(0:正常 1：双人验收，默认正常)")
	private Integer itemHealthIsDouble;

	// =================== 5.12 新增=========================================
	@ApiModelProperty(value = "(新)库区编号(逗号分隔列表)", position = -1)
	private String warehouseAreaCodes;
	@ApiModelProperty(value = "(新)商品整散分离标识(0整散合一/1整散分开)", position = -1)
	private String itemIsPieceOrCase;
	@ApiModelProperty(value = "(新)推荐库别编号(逗号分隔列表)", position = -1)
	private String warehouseAreaCategorys;

	// ===============5.17 新加字段=========================================
	@ApiModelProperty(value = "(新)收货类型", position = -1)
	private String rcvType;
	@ApiModelProperty(value = "(新)商品规格说明(冗余)", position = -1)
	private String specification;
	@ApiModelProperty(value = "(新)上市许可持有人(冗余)", position = -1)
	private String itemHealthListedLicensor;
	@ApiModelProperty(value = "(新)生产企业许可证号(冗余)", position = -1)
	private String itemHealthProduceLicense;
	@ApiModelProperty(value = "(新)存储分类(冗余)", position = -1)
	private String itemHealthStoreCategory;
	@ApiModelProperty(value = "(新)商品是否进口(冗余)", position = -1)
	private Integer itemIsImported;
	@ApiModelProperty(value = "(新)商品是否复检(0否1是)(冗余)", position = -1)
	private Integer itemIsRecheck;
	@ApiModelProperty(value = "(新)商品是否器械(0否1是)(冗余)", position = -1)
	private Integer isInstrument;

	@ApiModelProperty(value = "批次生产日期(列表展示用)", position = 9)
	private String batchProductionDate;
	@ApiModelProperty(value = "批次有效期(列表展示用)", position = 9)
	private String batchExpiredDate;
	@ApiModelProperty(value = "商品分类code名(展示用)", position = 9)
	private String categoryId;
	@ApiModelProperty(value = "仓库编码", position = 9)
	private String warehouseCode;

	// ========5.20新加==========================
	// 商品通用名
	@ApiModelProperty(value = "(新)商品通用名(冗余)", position = -2)
	private String itemHealthCommonName;
	// 商品是否锁定
	@ApiModelProperty(value = "(新)商品是否锁定(0否1是)(冗余)", position = -2)
	private Integer isLock;
	// 商品是否代管
	@ApiModelProperty(value = "(新)商品是否代管(0否1是)(冗余)", position = -2)
	private Integer isHosted;
	// 生产厂家id
	@ApiModelProperty(value = "(新)生产厂家id(冗余)", position = -2)
	private String manufacturerId;
	// 生产厂家code
	@ApiModelProperty(value = "(新)生产厂家id(冗余)", position = -2)
	private String manufacturerCode;
	// 生产厂家name
	@ApiModelProperty(value = "(新)生产厂家id(冗余)", position = -2)
	private String manufacturerName;
	@ApiModelProperty(value = "(新)入库预留保质天数(冗余)", position = -2)
	private BigDecimal inStoreDay;
	// 注册证号
	@ApiModelProperty(value = "(新)注册证号(冗余)", position = -2)
	private String itemHealthRegisterNbr;
	// 有效期天数(需转换)
	@ApiModelProperty(value = "(新)有效期天数(需转换)(冗余)", position = -2)
	private BigDecimal shelfLifeDays;
	// 保质期单位
	@ApiModelProperty(value = "(新)保质期单位(年/月/日,默认日)(冗余)", position = -2)
	private String shelfLifeUnit;
	// 是否特殊品
	@ApiModelProperty(value = "(新)商品是否特殊品(0否1是)(冗余)", position = -2)
	private Integer itemIsSpecial;
	// 是否无菌
	@ApiModelProperty(value = "(新)商品是否无菌(0否1是)(冗余)", position = -2)
	private Integer isBacteria;
	// 退货单位提供的商品售出期间储存、运输质量控制情况
	@ApiModelProperty(value = "(新)退货单位提供的商品售出期间储存、运输质量控制情况(冗余)", position = -2)
	private String returnInfo;
	// 退货冷藏品出库期质量
	@ApiModelProperty(value = "(新)退货冷藏品出库期质量(冗余)", position = -2)
	private String returnColdQuality;
	// 退货说明
	@ApiModelProperty(value = "(新)退货说明(冗余)", position = -2)
	private String returnExplain;

	@ApiModelProperty(value = "(新)选择的库区名称", position = -3)
	private String warehouseAreaName;
	@ApiModelProperty(value = "(新)选择的库区id", position = -3)
	private String warehouseAreaId;
	@ApiModelProperty(value = "(新)选择的库区编号", position = -3)
	private String warehouseAreaCode;
	@ApiModelProperty(value = "(新)选择的建筑物id", position = -3)
	private String bldgId;
	@ApiModelProperty(value = "(新)选择的建筑物名称", position = -3)
	private String bldgName;
	@ApiModelProperty(value = "(新)选择的建筑物编号", position = -3)
	private String bldgCode;

	// ==== 6.1 新加字段===========================================================
	@ApiModelProperty(value = "(新)受托生产企业id(供应商id)", position = -4)
	private String delegationPartyId;
	@ApiModelProperty(value = "(新)受托生产企业名称(供应商名称)", position = -4)
	private String delegationPartyName;

	// ==== 6.9 新加字段===========================================================
	@ApiModelProperty(value = "(新)显示生产日期", position = -4)
	private String showProductionDate;
	@ApiModelProperty(value = "(新)显示有效期至", position = -4)
	private String showExpiredDate;
	@ApiModelProperty(value = "(新)最小包装数", position = -4)
	private BigDecimal pieceConversionRate;
	@ApiModelProperty(value = "(新)推荐库房列表", position = -4)
	private String warehouseRoomCodes;

	// ============6.10新加字段======================================================
	@ApiModelProperty(value = "(新)asn单行明细备注", position = -4)
	private String asnLineRemark;

	// ============6.11新加字段======================================================
	@ApiModelProperty(value = "(新)商品助记码", position = -4)
	private String itemZjm;

	// 来源行id
	@ApiModelProperty(value = "(新)入库来源行id", position = -5)
	private String refLineId;

	@ApiModelProperty(value = "(新)处理原因", position = -5)
	private String handlingReason;
	@ApiModelProperty(value = "(新)不足原因", position = -5)
	private String defReason;
	@ApiModelProperty(value = "(新)处理措施", position = -5)
	private String handlingMeasure;
	@ApiModelProperty(value = "(新)不足原因补充", position = -5)
	private String defReasonExt;

	@ApiModelProperty(value = "(新)存储条件", position = -6)
	private String storeTemperature;

	@ApiModelProperty(value = "(新)来源单id(质检单)", position = -3)
	private String refBillId;
	@ApiModelProperty(value = "(新)来源单编号(质检单)", position = -3)
	private String refBillNo;

	@ApiModelProperty(value = "(新)复检结论1", position = -3)
	private String qcReResult;
	@ApiModelProperty(value = "(新)复检员1", position = -3)
	private String qcReOpcode;
	@ApiModelProperty(value = "(新)复检意见1", position = -3)
	private String qcReSuggestion;
	@ApiModelProperty(value = "(新)复检时间1", position = -3)
	private Date qcReDate;
	@ApiModelProperty(value = "(新)复检类型", position = -3)
	private String rqcReType;
	@ApiModelProperty(value = "(新)上一节点行id(用于记录上一次复检的行id)", position = -200)
	private String preNodeLineId;
	@ApiModelProperty(value = "(新)质检单id", position = -200)
	private String rqcHeaderId;

	@ApiModelProperty(value = "(新)商品批准文号", position = -1113)
	private String approvalNumber;
	@ApiModelProperty(value = "(新)是否委托生产(0否1是)", position = -4)
	private Integer isDelegate;
	@ApiModelProperty(value = "(新)质检结论(验收结论)", position = -250)
	private String rqcResult;
	@ApiModelProperty(value = "商品产地", position = -250)
	private String productionPlace;

	@ApiModelProperty(value = "(新)质检报告单号(关联批次)", position = -1114)
	private String rqcReportNo;

	@ApiModelProperty(value = "小数拆分标识(Y为允许,其他均不可)", position = -2)
	private String allowDecimal;
	@ApiModelProperty(value = "任务数据来源(0收货/1一次质检/2复检)", position = -200)
	private String sourceType;
	@ApiModelProperty(value = "包装件数(列表展示用)", position = 1)
	private BigDecimal oddCaseQty;
	@ApiModelProperty(value = "asn单号(ASN)", hidden = true)
	private String asnAutoBillNum;

	@ApiModelProperty(value = "供应商名称(查询字段)", position = 1)
	private String vendorName;

	public String getAsnAutoBillNum() {
		return asnAutoBillNum;
	}

	public void setAsnAutoBillNum(String asnAutoBillNum) {
		this.asnAutoBillNum = asnAutoBillNum;
	}

	public String getVendorName() {
		return vendorName;
	}

	public void setVendorName(String vendorName) {
		this.vendorName = vendorName;
	}
	public BigDecimal getOddCaseQty() {
		return oddCaseQty;
	}

	public void setOddCaseQty(BigDecimal oddCaseQty) {
		this.oddCaseQty = oddCaseQty;
	}

	public String getSourceType() {
		return sourceType;
	}

	public void setSourceType(String sourceType) {
		this.sourceType = sourceType;
	}

	public String getAllowDecimal() {
		return allowDecimal;
	}

	public void setAllowDecimal(String allowDecimal) {
		this.allowDecimal = allowDecimal;
	}

	public String getRqcReportNo() {
		return rqcReportNo;
	}

	public void setRqcReportNo(String rqcReportNo) {
		this.rqcReportNo = rqcReportNo;
	}

	public String getProductionPlace() {
		return productionPlace;
	}

	public void setProductionPlace(String productionPlace) {
		this.productionPlace = productionPlace;
	}

	public String getRqcResult() {
		return rqcResult;
	}

	public void setRqcResult(String rqcResult) {
		this.rqcResult = rqcResult;
	}

	public Integer getIsDelegate() {
		return isDelegate;
	}

	public void setIsDelegate(Integer isDelegate) {
		this.isDelegate = isDelegate;
	}

	public String getApprovalNumber() {
		return approvalNumber;
	}

	public void setApprovalNumber(String approvalNumber) {
		this.approvalNumber = approvalNumber;
	}

	public String getRqcHeaderId() {
		return rqcHeaderId;
	}

	public void setRqcHeaderId(String rqcHeaderId) {
		this.rqcHeaderId = rqcHeaderId;
	}

	public String getTaskRecheckId() {
		return taskRecheckId;
	}

	public void setTaskRecheckId(String taskRecheckId) {
		this.taskRecheckId = taskRecheckId;
	}

	public String getMtlOwnerId() {
		return mtlOwnerId;
	}

	public void setMtlOwnerId(String mtlOwnerId) {
		this.mtlOwnerId = mtlOwnerId;
	}

	public String getWarehouseId() {
		return warehouseId;
	}

	public void setWarehouseId(String warehouseId) {
		this.warehouseId = warehouseId;
	}

	public String getSrcBillType() {
		return srcBillType;
	}

	public void setSrcBillType(String srcBillType) {
		this.srcBillType = srcBillType;
	}

	public String getSrcHeaderId() {
		return srcHeaderId;
	}

	public void setSrcHeaderId(String srcHeaderId) {
		this.srcHeaderId = srcHeaderId;
	}

	public String getSrcLineId() {
		return srcLineId;
	}

	public void setSrcLineId(String srcLineId) {
		this.srcLineId = srcLineId;
	}

	public String getAsnHeaderId() {
		return asnHeaderId;
	}

	public void setAsnHeaderId(String asnHeaderId) {
		this.asnHeaderId = asnHeaderId;
	}

	public String getAsnLineId() {
		return asnLineId;
	}

	public void setAsnLineId(String asnLineId) {
		this.asnLineId = asnLineId;
	}

	public String getRcvHeaderId() {
		return rcvHeaderId;
	}

	public void setRcvHeaderId(String rcvHeaderId) {
		this.rcvHeaderId = rcvHeaderId;
	}

	public String getRcvLineId() {
		return rcvLineId;
	}

	public void setRcvLineId(String rcvLineId) {
		this.rcvLineId = rcvLineId;
	}

	public String getItemId() {
		return itemId;
	}

	public void setItemId(String itemId) {
		this.itemId = itemId;
	}

	public BigDecimal getQty() {
		return qty;
	}

	public void setQty(BigDecimal qty) {
		this.qty = qty;
	}

	public String getUom() {
		return uom;
	}

	public void setUom(String uom) {
		this.uom = uom;
	}

	public BigDecimal getConversionRate() {
		return conversionRate;
	}

	public void setConversionRate(BigDecimal conversionRate) {
		this.conversionRate = conversionRate;
	}

	public BigDecimal getPieceQty() {
		return pieceQty;
	}

	public void setPieceQty(BigDecimal pieceQty) {
		this.pieceQty = pieceQty;
	}

	public String getBatchId() {
		return batchId;
	}

	public void setBatchId(String batchId) {
		this.batchId = batchId;
	}

	public String getBatchNum() {
		return batchNum;
	}

	public void setBatchNum(String batchNum) {
		this.batchNum = batchNum;
	}

	public String getVendorBatchNum() {
		return vendorBatchNum;
	}

	public void setVendorBatchNum(String vendorBatchNum) {
		this.vendorBatchNum = vendorBatchNum;
	}

	public String getActiveFlag() {
		return activeFlag;
	}

	public void setActiveFlag(String activeFlag) {
		this.activeFlag = activeFlag;
	}

	public BigDecimal getRemainQty() {
		return remainQty;
	}

	public void setRemainQty(BigDecimal remainQty) {
		this.remainQty = remainQty;
	}

	public Integer getTimes() {
		return times;
	}

	public void setTimes(Integer times) {
		this.times = times;
	}

	public String getComments() {
		return comments;
	}

	public void setComments(String comments) {
		this.comments = comments;
	}

	public String getResult() {
		return result;
	}

	public void setResult(String result) {
		this.result = result;
	}

	public String getReason() {
		return reason;
	}

	public void setReason(String reason) {
		this.reason = reason;
	}

	public String getManufacturerLotNum() {
		return manufacturerLotNum;
	}

	public void setManufacturerLotNum(String manufacturerLotNum) {
		this.manufacturerLotNum = manufacturerLotNum;
	}

	public String getExecutor() {
		return executor;
	}

	public void setExecutor(String executor) {
		this.executor = executor;
	}

	public String getContainerCode() {
		return containerCode;
	}

	public void setContainerCode(String containerCode) {
		this.containerCode = containerCode;
	}

	public String getGoodsCaseCode() {
		return goodsCaseCode;
	}

	public void setGoodsCaseCode(String goodsCaseCode) {
		this.goodsCaseCode = goodsCaseCode;
	}

	public String getStatus() {
		return status;
	}

	public void setStatus(String status) {
		this.status = status;
	}

	public RcvLinesProfilesDTO getRcvLinesProfilesDTO() {
		return rcvLinesProfilesDTO;
	}

	public void setRcvLinesProfilesDTO(RcvLinesProfilesDTO rcvLinesProfilesDTO) {
		this.rcvLinesProfilesDTO = rcvLinesProfilesDTO;
	}

	public RcvLinesDTO getRcvLinesDTO() {
		return rcvLinesDTO;
	}

	public void setRcvLinesDTO(RcvLinesDTO rcvLinesDTO) {
		this.rcvLinesDTO = rcvLinesDTO;
	}

	public BigDecimal getOddPieceQty() {
		return oddPieceQty;
	}

	public void setOddPieceQty(BigDecimal oddPieceQty) {
		this.oddPieceQty = oddPieceQty;
	}

	public BigDecimal getCaseQty() {
		return caseQty;
	}

	public void setCaseQty(BigDecimal caseQty) {
		this.caseQty = caseQty;
	}

	public String getItemBarcode() {
		return itemBarcode;
	}

	public void setItemBarcode(String itemBarcode) {
		this.itemBarcode = itemBarcode;
	}

	public BigDecimal getItemVolume() {
		return itemVolume;
	}

	public void setItemVolume(BigDecimal itemVolume) {
		this.itemVolume = itemVolume;
	}

	public BigDecimal getItemWeight() {
		return itemWeight;
	}

	public void setItemWeight(BigDecimal itemWeight) {
		this.itemWeight = itemWeight;
	}

	public Integer getItemHealthTwovoflag() {
		return itemHealthTwovoflag;
	}

	public void setItemHealthTwovoflag(Integer itemHealthTwovoflag) {
		this.itemHealthTwovoflag = itemHealthTwovoflag;
	}

	public String getWarehouseName() {
		return warehouseName;
	}

	public void setWarehouseName(String warehouseName) {
		this.warehouseName = warehouseName;
	}

	public String getItemNum() {
		return itemNum;
	}

	public void setItemNum(String itemNum) {
		this.itemNum = itemNum;
	}

	public String getItemName() {
		return itemName;
	}

	public void setItemName(String itemName) {
		this.itemName = itemName;
	}

	public String getCategoryName() {
		return categoryName;
	}

	public void setCategoryName(String categoryName) {
		this.categoryName = categoryName;
	}

	public String getEnterpriseId() {
		return enterpriseId;
	}

	public void setEnterpriseId(String enterpriseId) {
		this.enterpriseId = enterpriseId;
	}

	public Date getGmtCreate() {
		return gmtCreate;
	}

	public void setGmtCreate(Date gmtCreate) {
		this.gmtCreate = gmtCreate;
	}

	public String getCreator() {
		return creator;
	}

	public void setCreator(String creator) {
		this.creator = creator;
	}

	public String getCreatorName() {
		return creatorName;
	}

	public void setCreatorName(String creatorName) {
		this.creatorName = creatorName;
	}

	public Date getGmtModified() {
		return gmtModified;
	}

	public void setGmtModified(Date gmtModified) {
		this.gmtModified = gmtModified;
	}

	public String getModifier() {
		return modifier;
	}

	public void setModifier(String modifier) {
		this.modifier = modifier;
	}

	public String getModifierName() {
		return modifierName;
	}

	public void setModifierName(String modifierName) {
		this.modifierName = modifierName;
	}

	public Integer getDataVersion() {
		return dataVersion;
	}

	public void setDataVersion(Integer dataVersion) {
		this.dataVersion = dataVersion;
	}

	public Integer getItemHealthIsDouble() {
		return itemHealthIsDouble;
	}

	public void setItemHealthIsDouble(Integer itemHealthIsDouble) {
		this.itemHealthIsDouble = itemHealthIsDouble;
	}

	public String getWarehouseAreaCodes() {
		return warehouseAreaCodes;
	}

	public void setWarehouseAreaCodes(String warehouseAreaCodes) {
		this.warehouseAreaCodes = warehouseAreaCodes;
	}

	public String getItemIsPieceOrCase() {
		return itemIsPieceOrCase;
	}

	public void setItemIsPieceOrCase(String itemIsPieceOrCase) {
		this.itemIsPieceOrCase = itemIsPieceOrCase;
	}

	public String getWarehouseAreaCategorys() {
		return warehouseAreaCategorys;
	}

	public void setWarehouseAreaCategorys(String warehouseAreaCategorys) {
		this.warehouseAreaCategorys = warehouseAreaCategorys;
	}

	public String getRcvType() {
		return rcvType;
	}

	public void setRcvType(String rcvType) {
		this.rcvType = rcvType;
	}

	public String getSpecification() {
		return specification;
	}

	public void setSpecification(String specification) {
		this.specification = specification;
	}

	public String getItemHealthListedLicensor() {
		return itemHealthListedLicensor;
	}

	public void setItemHealthListedLicensor(String itemHealthListedLicensor) {
		this.itemHealthListedLicensor = itemHealthListedLicensor;
	}

	public String getItemHealthProduceLicense() {
		return itemHealthProduceLicense;
	}

	public void setItemHealthProduceLicense(String itemHealthProduceLicense) {
		this.itemHealthProduceLicense = itemHealthProduceLicense;
	}

	public String getItemHealthStoreCategory() {
		return itemHealthStoreCategory;
	}

	public void setItemHealthStoreCategory(String itemHealthStoreCategory) {
		this.itemHealthStoreCategory = itemHealthStoreCategory;
	}

	public Integer getItemIsImported() {
		return itemIsImported;
	}

	public void setItemIsImported(Integer itemIsImported) {
		this.itemIsImported = itemIsImported;
	}

	public Integer getItemIsRecheck() {
		return itemIsRecheck;
	}

	public void setItemIsRecheck(Integer itemIsRecheck) {
		this.itemIsRecheck = itemIsRecheck;
	}

	public Integer getIsInstrument() {
		return isInstrument;
	}

	public void setIsInstrument(Integer isInstrument) {
		this.isInstrument = isInstrument;
	}

	public String getBatchProductionDate() {
		return batchProductionDate;
	}

	public void setBatchProductionDate(String batchProductionDate) {
		this.batchProductionDate = batchProductionDate;
	}

	public String getBatchExpiredDate() {
		return batchExpiredDate;
	}

	public void setBatchExpiredDate(String batchExpiredDate) {
		this.batchExpiredDate = batchExpiredDate;
	}

	public String getCategoryId() {
		return categoryId;
	}

	public void setCategoryId(String categoryId) {
		this.categoryId = categoryId;
	}

	public String getWarehouseCode() {
		return warehouseCode;
	}

	public void setWarehouseCode(String warehouseCode) {
		this.warehouseCode = warehouseCode;
	}

	public String getItemHealthCommonName() {
		return itemHealthCommonName;
	}

	public void setItemHealthCommonName(String itemHealthCommonName) {
		this.itemHealthCommonName = itemHealthCommonName;
	}

	public Integer getIsLock() {
		return isLock;
	}

	public void setIsLock(Integer isLock) {
		this.isLock = isLock;
	}

	public Integer getIsHosted() {
		return isHosted;
	}

	public void setIsHosted(Integer isHosted) {
		this.isHosted = isHosted;
	}

	public String getManufacturerId() {
		return manufacturerId;
	}

	public void setManufacturerId(String manufacturerId) {
		this.manufacturerId = manufacturerId;
	}

	public String getManufacturerCode() {
		return manufacturerCode;
	}

	public void setManufacturerCode(String manufacturerCode) {
		this.manufacturerCode = manufacturerCode;
	}

	public String getManufacturerName() {
		return manufacturerName;
	}

	public void setManufacturerName(String manufacturerName) {
		this.manufacturerName = manufacturerName;
	}

	public BigDecimal getInStoreDay() {
		return inStoreDay;
	}

	public void setInStoreDay(BigDecimal inStoreDay) {
		this.inStoreDay = inStoreDay;
	}

	public String getItemHealthRegisterNbr() {
		return itemHealthRegisterNbr;
	}

	public void setItemHealthRegisterNbr(String itemHealthRegisterNbr) {
		this.itemHealthRegisterNbr = itemHealthRegisterNbr;
	}

	public BigDecimal getShelfLifeDays() {
		return shelfLifeDays;
	}

	public void setShelfLifeDays(BigDecimal shelfLifeDays) {
		this.shelfLifeDays = shelfLifeDays;
	}

	public String getShelfLifeUnit() {
		return shelfLifeUnit;
	}

	public void setShelfLifeUnit(String shelfLifeUnit) {
		this.shelfLifeUnit = shelfLifeUnit;
	}

	public Integer getItemIsSpecial() {
		return itemIsSpecial;
	}

	public void setItemIsSpecial(Integer itemIsSpecial) {
		this.itemIsSpecial = itemIsSpecial;
	}

	public Integer getIsBacteria() {
		return isBacteria;
	}

	public void setIsBacteria(Integer isBacteria) {
		this.isBacteria = isBacteria;
	}

	public String getReturnInfo() {
		return returnInfo;
	}

	public void setReturnInfo(String returnInfo) {
		this.returnInfo = returnInfo;
	}

	public String getReturnColdQuality() {
		return returnColdQuality;
	}

	public void setReturnColdQuality(String returnColdQuality) {
		this.returnColdQuality = returnColdQuality;
	}

	public String getReturnExplain() {
		return returnExplain;
	}

	public void setReturnExplain(String returnExplain) {
		this.returnExplain = returnExplain;
	}

	public String getWarehouseAreaName() {
		return warehouseAreaName;
	}

	public void setWarehouseAreaName(String warehouseAreaName) {
		this.warehouseAreaName = warehouseAreaName;
	}

	public String getWarehouseAreaId() {
		return warehouseAreaId;
	}

	public void setWarehouseAreaId(String warehouseAreaId) {
		this.warehouseAreaId = warehouseAreaId;
	}

	public String getWarehouseAreaCode() {
		return warehouseAreaCode;
	}

	public void setWarehouseAreaCode(String warehouseAreaCode) {
		this.warehouseAreaCode = warehouseAreaCode;
	}

	public String getBldgId() {
		return bldgId;
	}

	public void setBldgId(String bldgId) {
		this.bldgId = bldgId;
	}

	public String getBldgName() {
		return bldgName;
	}

	public void setBldgName(String bldgName) {
		this.bldgName = bldgName;
	}

	public String getBldgCode() {
		return bldgCode;
	}

	public void setBldgCode(String bldgCode) {
		this.bldgCode = bldgCode;
	}

	public String getDelegationPartyId() {
		return delegationPartyId;
	}

	public void setDelegationPartyId(String delegationPartyId) {
		this.delegationPartyId = delegationPartyId;
	}

	public String getDelegationPartyName() {
		return delegationPartyName;
	}

	public void setDelegationPartyName(String delegationPartyName) {
		this.delegationPartyName = delegationPartyName;
	}

	public String getShowProductionDate() {
		return showProductionDate;
	}

	public void setShowProductionDate(String showProductionDate) {
		this.showProductionDate = showProductionDate;
	}

	public String getShowExpiredDate() {
		return showExpiredDate;
	}

	public void setShowExpiredDate(String showExpiredDate) {
		this.showExpiredDate = showExpiredDate;
	}

	public BigDecimal getPieceConversionRate() {
		return pieceConversionRate;
	}

	public void setPieceConversionRate(BigDecimal pieceConversionRate) {
		this.pieceConversionRate = pieceConversionRate;
	}

	public String getWarehouseRoomCodes() {
		return warehouseRoomCodes;
	}

	public void setWarehouseRoomCodes(String warehouseRoomCodes) {
		this.warehouseRoomCodes = warehouseRoomCodes;
	}

	public String getAsnLineRemark() {
		return asnLineRemark;
	}

	public void setAsnLineRemark(String asnLineRemark) {
		this.asnLineRemark = asnLineRemark;
	}

	public String getItemZjm() {
		return itemZjm;
	}

	public void setItemZjm(String itemZjm) {
		this.itemZjm = itemZjm;
	}

	public String getRefLineId() {
		return refLineId;
	}

	public void setRefLineId(String refLineId) {
		this.refLineId = refLineId;
	}

	public String getHandlingReason() {
		return handlingReason;
	}

	public void setHandlingReason(String handlingReason) {
		this.handlingReason = handlingReason;
	}

	public String getDefReason() {
		return defReason;
	}

	public void setDefReason(String defReason) {
		this.defReason = defReason;
	}

	public String getHandlingMeasure() {
		return handlingMeasure;
	}

	public void setHandlingMeasure(String handlingMeasure) {
		this.handlingMeasure = handlingMeasure;
	}

	public String getDefReasonExt() {
		return defReasonExt;
	}

	public void setDefReasonExt(String defReasonExt) {
		this.defReasonExt = defReasonExt;
	}

	public String getStoreTemperature() {
		return storeTemperature;
	}

	public void setStoreTemperature(String storeTemperature) {
		this.storeTemperature = storeTemperature;
	}

	public String getRefBillId() {
		return refBillId;
	}

	public void setRefBillId(String refBillId) {
		this.refBillId = refBillId;
	}

	public String getRefBillNo() {
		return refBillNo;
	}

	public void setRefBillNo(String refBillNo) {
		this.refBillNo = refBillNo;
	}

	public String getQcReResult() {
		return qcReResult;
	}

	public void setQcReResult(String qcReResult) {
		this.qcReResult = qcReResult;
	}

	public String getQcReOpcode() {
		return qcReOpcode;
	}

	public void setQcReOpcode(String qcReOpcode) {
		this.qcReOpcode = qcReOpcode;
	}

	public String getQcReSuggestion() {
		return qcReSuggestion;
	}

	public void setQcReSuggestion(String qcReSuggestion) {
		this.qcReSuggestion = qcReSuggestion;
	}

	public Date getQcReDate() {
		return qcReDate;
	}

	public void setQcReDate(Date qcReDate) {
		this.qcReDate = qcReDate;
	}

	public String getRqcReType() {
		return rqcReType;
	}

	public void setRqcReType(String rqcReType) {
		this.rqcReType = rqcReType;
	}

	public String getPreNodeLineId() {
		return preNodeLineId;
	}

	public void setPreNodeLineId(String preNodeLineId) {
		this.preNodeLineId = preNodeLineId;
	}

}
