package com.prolog.upcloud.base.wh.operate.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;

/**
 * EIS盘点审核信息推送
 *
 * @author 鄢菁KingYan
 * @date 2022/07/13
 */
@Data
public class EisInventoryRequestVO {
    @ApiModelProperty(value = "仓库编号", required = true)
    private String warehouseId;

    @ApiModelProperty(value = "货主ID", required = true)
    private String ownerId;

    @ApiModelProperty(value = "单据编号", required = true, notes = "盘点单号")
    private String billNo;

    @ApiModelProperty(value = "行号", required = true, notes = "有任务取任务的行号，无任务的业务取单明细的行号")
    private String seqNo;

    @ApiModelProperty(value = "商品编号", required = true)
    private String itemCode;

    @ApiModelProperty(value = "商品名称", required = false)
    private String itemName;

    @ApiModelProperty(value = "批号", required = true, notes = "盘点到托盘，批号级别")
    private String lotNo;

    @ApiModelProperty(value = "生产日期", required = true)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date yieldDate;

    @ApiModelProperty(value = "有效日期", required = true)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date expDate;

    @ApiModelProperty(value = "库存数量", required = true)
    private BigDecimal inventoryQty;

    @ApiModelProperty(value = "实盘数量", required = true)
    private BigDecimal countQty;

    @ApiModelProperty(value = "容器编号", required = true, notes = "盘点到托盘，批号级别")
    private String containerNo;

    @ApiModelProperty(value = "WMS货位", required = true, notes = "大货位：ASRS01")
    private String wmslocationCode;

    @ApiModelProperty(value = "指令ID", required = true, notes = "每行唯一")
    private String msgId;
}
